// -*- C++ -*-
/*!
 * @file InPort.h
 * @brief InPort template class
 * @date $Date: 2007-12-31 03:08:03 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2003-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: InPort.h 1351 2009-05-19 23:54:52Z n-ando $
 *
 */

#ifndef RtcInPort_h
#define RtcInPort_h

#include <string>
#include <vector>
#include <iostream>

#include <coil/TimeValue.h>
#include <coil/Time.h>
#include <coil/OS.h>

#include <rtm/RTC.h>
#include <rtm/Typename.h>
#include <rtm/InPortBase.h>
#include <rtm/CdrBufferBase.h>
#include <rtm/PortCallBack.h>
#include <rtm/InPortConnector.h>

namespace RTC
{
  /*!
   * @if jp
   *
   * @class InPort
   *
   * @brief InPort ev[gNX
   * 
   * InPort ̎ł InPort<T> ̃ev[gNXB
   * <T> BasicDataType.idl ɂĒ`Ă^ŁAoƂ
   * Time ^ tm ,  T^ data \̂łȂĂ͂ȂȂB
   * InPort ͓ɃOobt@AO瑗Mꂽf[^
   * ̃Oobt@Ɋi[BOobt@̃TCY̓ftHg64
   * ȂĂ邪ARXgN^ɂTCYw肷邱ƂłB
   * f[^̓tOɂĖǁAǏԂǗAisNew(), write(), read(),
   * isFull(), isEmpty() ̃\bhɂnhO邱ƂłB
   *
   * @since 0.2.0
   *
   * @else
   *
   * @class InPort
   *
   * @brief InPort template class
   *
   * This is a template class that implements InPort.
   * <T> is the type defined in BasicDataType.idl and must be the structure which
   * has both Time type tm and type-T data as a member. InPort has a ring
   * buffer internally, and stores the received data externally in this buffer
   * one by one. The size of ring buffer can be specified according to 
   * the argument of constructor, though the default size is 64. Unread
   * data and data which is already read are managed with the flag, and the
   * data can be handled by the isNew(), write(), read(), isFull() and isEmpty()
   * method etc. 
   *
   * @since 0.2.0
   *
   * @endif
   */
  template <class DataType>
  class InPort
    : public InPortBase
  {
  public:
    DATAPORTSTATUS_ENUM
    /*!
     * @if jp
     *
     * @brief RXgN^
     *
     * RXgN^B
     * p[^Ƃė^ T ^̕ϐɃoChB
     *
     * @param name InPort BInPortBase:name() ɂQƂB
     * @param value  InPort ɃoCh T ^̕ϐ
     * @param bufsize InPort ̃Oobt@̃obt@(ftHgl:64)
     * @param read_block ǍubNtOB
     *        f[^Ǎɖǃf[^ȂꍇÃf[^M܂ŃubN
     *        ǂݒ(ftHgl:false)
     * @param write_block ubNtOB
     *        f[^Ƀobt@tłꍇAobt@ɋ󂫂ł
     *        ܂ŃubN邩ǂݒ(ftHgl:false)
     * @param read_timeout ǍubNw肵ĂȂꍇ́Af[^ǎ^C
     *        AEg(~b)(ftHgl:0)
     * @param write_timeout ubNw肵ĂȂꍇ́Af[^^C
     *        AEg(~b)(ftHgl:0)
     *
     * @else
     *
     * @brief A constructor.
     *
     * constructor.
     * This is bound to type-T variable given as a parameter.
     *
     * @param name A name of the InPort. This name is referred by
     *             InPortBase::name().
     * @param value type-T variable that is bound to this InPort.
     * @param bufsize Buffer length of internal ring buffer of InPort
     *                (The default value:64)
     * @param read_block Flag of reading block.
     *                   When there are not unread data at reading data,
     *                   set whether to block data until receiving the next 
     *                   data. (The default value:false)
     * @param write_block Flag of writing block.
     *                    If the buffer was full at writing data, set whether 
     *                    to block data until the buffer has space. 
     *                    (The default value:false)
     * @param read_timeout Data reading timeout time (millisecond) 
     *                     when not specifying read blocking.
     *                     (The default value:0)
     * @param write_timeout Data writing timeout time (millisecond)
     *                      when not specifying writing block.
     *                      (The default value:0)
     *
     * @endif
     */
    InPort(const char* name, DataType& value,
	   int bufsize=64, 
	   bool read_block = false, bool write_block = false,
	   int read_timeout = 0, int write_timeout = 0)
      :	InPortBase(name, toTypename<DataType>()),
        m_name(name), m_value(value),
	m_readBlock(read_block),   m_readTimeout(read_timeout),
	m_writeBlock(write_block), m_writeTimeout(write_timeout),
	m_OnWrite(NULL), m_OnWriteConvert(NULL),
	m_OnRead(NULL),  m_OnReadConvert(NULL),
	m_OnOverflow(NULL), m_OnUnderflow(NULL)
    {
    }
    
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * fXgN^B
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~InPort(void){};

    /*!
     * @if jp
     *
     * @brief |[ĝ擾B
     *
     * |[ĝ擾B
     *
     * @return |[g
     *
     * @else
     *
     * @brief Get port name
     *
     * Get port name.
     *
     * @return The port name
     *
     * @endif
     */
    virtual const char* name()
    {
      return m_name.c_str();
    }

    
    /*!
     * @if jp
     *
     * @brief ŐVf[^݂邩mF
     * 
     * InPortɖǂ̍ŐVf[^Ă邩boollŕԂB
     * InPortڑ̏ꍇAѐڑRlN^̃obt@Empty
     * ̏ꍇɂfalseԂB
     *
     * @return true ǂ̍ŐVf[^݂
     *         false ڑ܂̓obt@Ƀf[^݂ȂB
     * 
     * @else
     *
     * @brief Check whether the data is newest
     * 
     * Check whether the data stored at a current buffer position is newest.
     *
     * @return Newest data check result
     *         ( true:Newest data. Data has not been readout yet.
     *          false:Past dataDData has already been readout.)
     * 
     * @endif
     */
    virtual bool isNew()
    {
      RTC_TRACE(("isNew()"));
      if (m_connectors.size() == 0)
        {
          RTC_DEBUG(("no connectors"));
          return false;
        }
      int r(m_connectors[0]->getBuffer()->readable());
      if (r > 0)
        {
          RTC_DEBUG(("isNew() = true, readable data: %d", r));
          return true;
        }
      
      RTC_DEBUG(("isNew() = false, no readable data"));
      return false;
    }

    /*!
     * @if jp
     *
     * @brief obt@󂩂ǂmF
     * 
     * InPort̃obt@󂩂ǂ bool lŕԂB
     * ̏ꍇ true, ǃf[^ꍇ false ԂB
     *
     * @return true  obt@͋
     *         false obt@ɖǃf[^
     * 
     * @else
     *
     * @brief Check whether the data is newest
     * 
     * Check whether the data stored at a current buffer position is newest.
     *
     * @return Newest data check result
     *         ( true:Newest data. Data has not been readout yet.
     *          false:Past dataDData has already been readout.)
     * 
     * @endif
     */
    virtual bool isEmpty()
    {
      RTC_TRACE(("isEmpty()"));

      if (m_connectors.size() == 0)
        {
          RTC_DEBUG(("no connectors"));
          return true;
        }
      int r(m_connectors[0]->getBuffer()->readable());
      if (r == 0)
        {
          RTC_DEBUG(("isEmpty() = true, buffer is empty"));
          return true;
        }
      
      RTC_DEBUG(("isEmpty() = false, data exists in the buffer"));
      return false;
    }

    /*!
     * @if jp
     *
     * @brief DataPort lǂݏo
     *
     * InPortɏ܂ꂽf[^ǂ݂Bڑ0A܂̓obt@
     * f[^܂ĂȂԂœǂ݂ꍇ̖߂l͕słB
     * obt@̏Ԃ̂ƂA
     * Oɐݒ肳ꂽ[h (readback, do_nothing, block) ɉāA
     * ȉ̂悤ȓB
     *
     * - readback: Ō̒lǂ݂ȂB
     *
     * - do_nothing: Ȃ
     *
     * - block: ubNB^CAEgݒ肳Ăꍇ́A
     *       ^CAEg܂ő҂B
     *
     * obt@̏Ԃł́AInPortɃoChꂽϐ̒lԂB
     * āAǂݏoɂ͕slԂ\B
     * ̊֐𗘗pۂɂ́A
     *
     * - isNew(), isEmpty() ƕpAOɃobt@Ԃ`FbNB
     * 
     * - ǂݏoɕslԂȂ悤ɃoChϐOɏ
     * 
     * - ReturnCode read(DataType& data) ֐̗pB
     *
     * Ƃ]܂B
     *
     * eR[obN֐͈ȉ̂悤ɌĂяoB
     * - OnRead: read() ֐Ă΂ۂɕKĂ΂B
     * 
     * - OnReadConvert: f[^̓ǂݏoꍇAǂ݂f[^
     *       ƂOnReadConvertĂяoA߂lread()߂l
     *       ƂĕԂB
     *
     * - OnEmpty: obt@̂߃f[^̓ǂݏoɎsꍇĂяoB
     *        OnEmpty ̖߂l read() ̖߂lƂĕԂB
     *
     * - OnBufferTimeout: f[^t[^Push^̏ꍇɁAǂݏo
     *        ^CAEĝ߂Ƀf[^̓ǂݏoɎsꍇɌĂ΂B
     *
     * - OnRecvTimeout: f[^t[^Pull^̏ꍇɁAǂݏo^CAEg
     *        ̂߂Ƀf[^ǂݏoɎsꍇɌĂ΂B
     *
     * - OnReadError: LȊO̗Rœǂ݂ɎsꍇɌĂ΂B
     *        RƂẮAobt@ݒ̕sAO̔Ȃǂl
     *        ʏ͋N肦Ȃ߃oỎ\B
     *
     * @return ǂݏof[^
     *
     * @else
     *
     * @brief Readout the value from DataPort
     *
     * Readout the value from DataPort
     *
     * - When Callback functor OnRead is already set, OnRead will be invoked
     *   before reading from the buffer held by DataPort.
     * - When the buffer held by DataPort can detect the underflow,
     *   and when it detected the underflow at reading, callback functor
     *   OnUnderflow will be invoked.
     * - When callback functor OnReadConvert is already set, the return value of
     *   operator() of OnReadConvert will be the return value of read().
     * - When timeout of reading is already set by setReadTimeout(),
     *   it waits for only timeout time until the state of the buffer underflow
     *   is reset, and if OnUnderflow is already set, this will be invoked to 
     *   return.
     *
     * @return Readout data
     *
     * @endif
     */
    DataType read()
    {
      RTC_TRACE(("DataType read()"))

      if (m_OnRead != NULL) 
        {
          (*m_OnRead)();
          RTC_TRACE(("OnRead called"))
        }

      if (m_connectors.size() == 0)
        {
          RTC_DEBUG(("no connectors"));
          return m_value;
        }

      cdrMemoryStream cdr;

      ReturnCode ret(m_connectors[0]->read(cdr));
      if (ret == PORT_OK)
        {
          RTC_DEBUG(("data read succeeded"));
          m_value <<= cdr;
          if (m_OnReadConvert != 0) 
            {
              m_value = (*m_OnReadConvert)(m_value);
              RTC_DEBUG(("OnReadConvert called"));
              return m_value;
            }
          return m_value;
        }
      else if (ret == BUFFER_EMPTY)
        {
          RTC_WARN(("buffer empty"));
          return m_value;
        }
      else if (ret == BUFFER_TIMEOUT)
        {
          RTC_WARN(("buffer read timeout"));
          return m_value;
        }
      RTC_ERROR(("unknown retern value from buffer.read()"));
      return m_value;
    }
    
    /*!
     * @if jp
     *
     * @brief oChꂽ T ^̕ϐ InPort obt@̍ŐVlǂݍ
     *
     * oChꂽ T ^̃f[^ InPort ̍ŐVlǂݍށB
     * RXgN^ T ^̕ϐ InPort oChĂȂ΂ȂȂB
     * ̃\bh̓|[tBbNɎgp鎖OƂĂ邽߁A
     * ^ɈˑȂA߂lƂȂĂB
     *
     * @else
     *
     * @brief Read the newly value to type-T variable which is bound to InPort's
     *        buffer.
     *
     * Read the newly value to type-T data which is bound to InPort's buffer.
     * The type-T variable must be bound to InPort in constructor.
     * Since this method assumes to be used for polymorphic,
     * its argument and the return value do not depend on type.
     *
     * @endif
     */
    virtual void update()
    {
      this->read();
    };
    
    /*!
     * @if jp
     *
     * @brief T ^̃f[^ InPort ̍ŐVlf[^ǂݍ
     *
     * InPort ɐݒ肳ĂŐVf[^ǂݍ݁A
     * w肳ꂽf[^ϐɐݒ肷B
     *
     * @param rhs InPort obt@lǂݍ T ^ϐ
     *
     * @else
     *
     * @brief Read the newly value data in InPort to type-T variable
     *
     * Read the newly data set in InPort and set to specified data variable.
     *
     * @param rhs The type-T variable to read from InPort's buffer
     *
     * @endif
     */
    void operator>>(DataType& rhs)
    {
      rhs = read();
      return;
    }
    
    /***
     * @if jp
     *
     * @brief ǂ̐Vf[^擾
     *
     * @else
     *
     * @brief Get number of new unread data
     *
     * @endif
     */
    /*
      virtual int getNewDataLen()
      {
      return m_buffer->new_data_len();
      }
    */
    
    /***
     * @if jp
     *
     * @brief ǂ̐Vf[^擾
     *
     * @else
     *
     * @brief Get new unread data
     *
     * @endif
     */
    /*
      virtual std::vector<T> getNewList()
      {
      return m_buffer.get_new_list();
      }
    */
    
    /***
     * @if jp
     *
     * @brief ǂ̐Vf[^t(V->)Ŏ擾
     *
     * @else
     *
     * @brief Get new unread data backwards by date (new->old)
     *
     * @endif
     */
    /*
      virtual std::vector<T> getNewListReverse()
      {
      return m_buffer.get_new_rlist();
      }
    */
    
    /*!
     * @if jp
     *
     * @brief InPort obt@փf[^͎̃R[obN̐ݒ
     *
     * InPort obt@Ƀf[^putꂽƂɌĂ΂R[obN
     * IuWFNgݒ肷Bݒ肳R[obNIuWFNg
     * InPort<DataType>::OnPutNXpA const DataType& A
     * ߂l void  operator() ֐ĂKvB
     *
     * struct MyOnPutCallback : public InPort<DataType> {<br>
     *   void operator()(const DataType data) {<br>
     *     <br>
     *   }<br>
     * };<br>
     * ̂悤ɃR[obNIuWFNgA<br>
     * <br> 
     * m_inport.setOnPut(new MyOnPutCallback());<br>
     * ̂悤ɃR[obNIuWFNgZbgB
     *
     * @param on_write OnWrite&lt;DataType&gt;^̃IuWFNg
     *
     * @else
     *
     * @brief Set callback for inputting data into the InPort buffer.
     *
     * Set the callback object invoked when data was put in the InPort's buffer.
     * The callback object which was set inherits InPort<DataType>::OnPut class,
     * and the method operator(), which has the argument "const DataType&"
     * and the return value "void", need to be implemented.
     *
     * Callback object should be implemented as follow:<br>
     * struct MyOnPutCallback : public InPort<DataType> {<br>
     *   void operator()(const DataType data) {<br>
     *     Operation<br>
     *   }<br>
     * };<br>
     * and should be set as follow:
     * <br> 
     * m_inport.setOnPut(new MyOnPutCallback());<br>
     *
     * @param on_write OnWrite&lt;DataType&gt; type object
     *
     * @endif
     */
    inline void setOnWrite(OnWrite<DataType>* on_write)
    {
      m_OnWrite = on_write;
    }
    
    /*!
     * @if jp
     *
     * @brief InPort obt@փf[^ݎ̃R[obN̐ݒ
     *
     * InPort obt@Ƀf[^܂鎞ɌĂ΂R[obN
     * IuWFNgݒ肷Bobt@ɂ̓R[obNIuWFNg
     * ߂lݒ肳B
     * 
     * @param on_wconvert OnWriteConvert&lt;DataType&gt;^̃IuWFNg
     *
     * @else
     *
     * @brief Set callback when data is written into the InPort buffer
     *
     * Set the callback object that is invoked when data is written into 
     * the InPort's buffer. The return value of the callback object is set to
     * the buffer.
     * 
     * @param on_wconvert OnWriteConvert&lt;DataType&gt; type object
     *
     * @endif
     */
    inline void setOnWriteConvert(OnWriteConvert<DataType>* on_wconvert)
    {
      m_OnWriteConvert = on_wconvert;
    }
    
    /*!
     * @if jp
     *
     * @brief InPort obt@փf[^ǂݍݎ̃R[obN̐ݒ
     *
     * InPort obt@f[^ǂݍ܂钼OɌĂ΂R[obN
     * IuWFNgݒ肷B
     * 
     * @param on_read OnRead&lt;DataType&gt;^̃IuWFNg
     *
     * @else
     *
     * @brief Set callback when data is read from the InPort buffer
     *
     * Set the callback object that is invoked right before data is read from 
     * the InPort's buffer
     * 
     * @param on_read OnRead&lt;DataType&gt; type object
     *
     * @endif
     */
    inline void setOnRead(OnRead<DataType>* on_read)
    {
      m_OnRead = on_read;
    }
    
    /*!
     * @if jp
     *
     * @brief InPort obt@փf[^ǂݏõR[obN̐ݒ
     *
     * InPort obt@f[^ǂݏoۂɌĂ΂R[obN
     * IuWFNgݒ肷BR[obNIuWFNg̖߂lread()\bh
     * ̌ďoʂƂȂB
     * 
     * @param on_rconvert OnReadConvert&lt;DataType&gt;^̃IuWFNg
     *
     * @else
     *
     * @brief Set callback when data is readout to the InPort buffer
     *
     * Set the callback object that is invoked when data is readout to
     * the InPort's buffer. The return value of callback object is the return
     * result of the read() method.
     * 
     * @param on_rconvert OnReadConvert&lt;DataType&gt; type object
     *
     * @endif
     */
    inline void setOnReadConvert(OnReadConvert<DataType>* on_rconvert)
    {
      m_OnReadConvert = on_rconvert;
    }
    
    /*!
     * @if jp
     *
     * @brief InPort obt@փobt@I[o[t[̃R[obN̐ݒ
     *
     * InPort obt@Ńobt@I[o[t[oꂽۂɌĂяo
     * R[obNIuWFNgݒ肷B
     * 
     * @param on_overflow OnOverflow&lt;DataType&gt;^̃IuWFNg
     *
     * @else
     *
     * @brief Set callback when the InPort buffer occurs overflow
     *
     * Set the callback object that is invoked when the buffer overflow was
     * detected in the InPort's buffer.
     * 
     * @param on_overflow OnOverflow&lt;DataType&gt; type object
     *
     * @endif
     */
    inline void setOnOverflow(OnOverflow<DataType>* on_overflow)
    {
      m_OnOverflow = on_overflow;
    }
    
    /*!
     * @if jp
     *
     * @brief InPort obt@փobt@A_[t[̃R[obN̐ݒ
     *
     * InPort obt@Ńobt@A_[t[oꂽۂɌĂяo
     * R[obNIuWFNgݒ肷B
     * 
     * @param on_underflow OnUnderflow&lt;DataType&gt;^̃IuWFNg
     *
     * @else
     *
     * @brief Set callback when the InPort buffer occurs underflow.
     *
     * Set the callback object that is invoked when the buffer underflow was
     * detected in the InPort's buffer.
     * 
     * @param on_underflow OnUnderflow&lt;DataType&gt; type object
     *
     * @endif
     */
    inline void setOnUnderflow(OnUnderflow<DataType>* on_underflow)
    {
      m_OnUnderflow = on_underflow;
    }
    
  private:
    std::string m_typename;
    /*!
     * @if jp
     * @brief |[g
     * @else
     * @berif Port's name
     * @endif
     */
    std::string m_name;
    
    /*!
     * @if jp
     * @brief oCh T ^̕ϐւ̎Q
     * @else
     * @brief The reference to type-T value bound this OutPort
     * @endif
     */
    DataType& m_value;
    
    bool m_readBlock;
    long int m_readTimeout;
    bool m_writeBlock;
    long int m_writeTimeout;
    
    /*!
     * @if jp
     * @brief OnWrite R[obNt@N^ւ̃|C^
     * @else
     * @brief Pointer to OnWrite callback functor
     * @endif
     */
    OnWrite<DataType>* m_OnWrite;
    
    /*!
     * @if jp
     * @brief OnWriteConvert R[obNt@N^ւ̃|C^
     * @else
     * @brief Pointer to OnWriteConvert callback functor
     * @endif
     */
    OnWriteConvert<DataType>* m_OnWriteConvert;
    
    /*!
     * @if jp
     * @brief OnRead R[obNt@N^ւ̃|C^
     * @else
     * @brief Pointer to OnRead callback functor
     * @endif
     */
    OnRead<DataType>* m_OnRead;
    
    /*!
     * @if jp
     * @brief OnReadConvert R[obNt@N^ւ̃|C^
     * @else
     * @brief Pointer to OnReadConvert callback functor
     * @endif
     */
    OnReadConvert<DataType>* m_OnReadConvert;
    
    /*!
     * @if jp
     * @brief OnOverflow R[obNt@N^ւ̃|C^
     * @else
     * @brief Pointer to OnOverflow callback functor
     * @endif
     */
    OnOverflow<DataType>* m_OnOverflow;
    
    /*!
     * @if jp
     * @brief OnUnderflow R[obNt@N^ւ̃|C^
     * @else
     * @brief Pointer to OnUnderflow callback functor
     *
     * @endif
     */
    OnUnderflow<DataType>* m_OnUnderflow;

    
  };
}; // End of namesepace RTM

#endif // RtcInPort_h
