// -*- C++ -*-
/*!
 * @file InPortBase.h
 * @brief RTC::Port implementation for InPort
 * @date $Date: 2008-01-13 15:06:40 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2009
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: InPortBase.h 1351 2009-05-19 23:54:52Z n-ando $
 *
 */

#ifndef RTC_INPORTBASE_H
#define RTC_INPORTBASE_H

#include <rtm/PortBase.h>
#include <rtm/DataPortStatus.h>
#include <rtm/CdrBufferBase.h>

/*!
 * @if jp
 * @namespace RTC
 *
 * @brief RTR|[lg
 *
 * @else
 *
 * @namespace RTC
 *
 * @brief RT-Component
 *
 * @endif
 */
namespace RTC
{
  class InPortProvider;
  class OutPortConsumer;
  class InPortConnector;

  /*!
   * @if jp
   * @class InPortBase
   * @brief InPort p Port
   *
   * f[^̓|[g̎NXB
   *
   * @since 0.4.0
   *
   * @else
   * @class InPortBase
   * @brief Port for InPort
   *
   * This is an implementation class for the data input port.
   *
   * @since 0.4.0
   *
   * @endif
   */
  class InPortBase
    : public PortBase, public DataPortStatus
  {
  public:
    DATAPORTSTATUS_ENUM

    typedef std::vector<InPortConnector*> ConnectorList;

    /*!
     * @if jp
     * @brief RXgN^
     *
     * RXgN^
     *
     * @param name |[g
     * @param inport Yf[^̓|[gɊ֘AtInPortIuWFNg
     *               InPortIuWFNgňf[^^Aobt@^Cvw肷
     * @param prop |[gݒpvpeB
     *
     * @else
     * @brief Constructor
     *
     * Constructor
     *
     * @param name Port name
     * @param inport InPort object that is associated with this data input port.
     *               Specify also the data type and the buffer type used in 
     *               the InPort object.
     * @param prop Property for setting ports
     *
     * @endif
     */
    InPortBase(const char* name, const char* data_type);
    
    /*!
     * @if jp
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~InPortBase(void);
    
    coil::Properties& properties();

    void init();

    /*!
     * @if jp
     *
     * @brief InPort activates 
     *
     * InPort activate B
     *
     * @else
     *
     * @brief Activate all Port interfaces
     *
     * This operation activate all interfaces that is registered in the
     * ports.
     *
     * @endif
     */
    virtual void activateInterfaces();

    /*!
     * @if jp
     *
     * @brief SĂ Port ̃C^[tF[X deactivates 
     *
     * Port ɓo^ĂSẴC^[tF[X deactivate B
     *
     * @else
     *
     * @brief Deactivate all Port interfaces
     *
     * This operation deactivate all interfaces that is registered in the
     * ports.
     *
     * @endif
     */
    virtual void deactivateInterfaces();

  protected:
    /*!
     * @if jp
     * @brief InterfaceJ
     *
     * InterfaceJB
     *  ConnectorProfile Ɋi[Ă dataflow_type  push ^
     * ̏ꍇ́Aw肳ꂽ interface_type  InPortProvider Ɋւ
     *  ConnectorProfile::properties ɏ݌Ăяoɖ߂B
     *
     *  dataport.dataflow_type
     *
     * @param connector_profile RlN^vt@C
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     * @brief Publish interface information
     *
     * Publish interface information.
     * Assign the Provider information that owned by this port
     * to ConnectorProfile#properties
     *
     * @param connector_profile The connector profile
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t
    publishInterfaces(ConnectorProfile& connector_profile);
    
    /*!
     * @if jp
     * @brief Interfaceɐڑ
     *
     * InterfaceɐڑB
     * PortLConsumerɓKProviderɊւ 
     * ConnectorProfile#properties 璊oA
     * ConsumerCORBAIuWFNgQƂݒ肷B
     *
     * @param connector_profile RlN^Evt@C
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     * @brief Subscribe to the interface
     *
     * Subscribe to interface.
     * Derive Provider information that matches Consumer owned by the Port 
     * from ConnectorProfile#properties and 
     * set the Consumer to the reference of the CORBA object.
     *
     * @param connector_profile The connector profile
     *
     * @return ReturnCode_t The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t
    subscribeInterfaces(const ConnectorProfile& connector_profile);
    
    /*!
     * @if jp
     * @brief Interfaceւ̐ڑ
     *
     * Interfaceւ̐ڑB
     * ^ꂽConnectorProfileɊ֘AConsumerɐݒ肳ꂽSĂObject
     * ڑB
     *
     * @param connector_profile RlN^Evt@C
     *
     * @else
     * @brief Disconnect the interface connection
     *
     * Disconnect the interface connection.
     * Release all objects set in Consumer associated with 
     * given ConnectorProfile and unscribe the interface.
     *
     * @param connector_profile The connector profile
     *
     * @endif
     */
    virtual void
    unsubscribeInterfaces(const ConnectorProfile& connector_profile);


    /*!
     * @if jp
     * @brief InPort provider ̏
     * @else
     * @brief InPort provider initialization
     * @endif
     */
    void initProviders();

    /*!
     * @if jp
     * @brief OutPort consumer ̏
     * @else
     * @brief OutPort consumer initialization
     * @endif
     */
    void initConsumers();

    /*!
     * @if jp
     * @brief InPort provider ̐
     *
     * InPortProvider 𐶐A ConnectorProfile ɌJB
     * Ɏsꍇ 0 ԂB
     *
     * @else
     * @brief InPort provider creation
     * @endif
     */
    InPortProvider*
    createProvider(ConnectorProfile& cprof, coil::Properties& prop);

    /*!
     * @if jp
     * @brief OutPort consumer ̐
     *
     * OutPortConsumer 𐶐B
     * Ɏsꍇ 0 ԂB
     *
     * @else
     * @brief InPort provider creation
     * @endif
     */
    OutPortConsumer*
    createConsumer(const ConnectorProfile& cprof, coil::Properties& prop);

    /*!
     * @if jp
     * @brief InPortPushConnector ̐
     *
     * Connector 𐶐A m_connectors ɕۑB
     * Ɏsꍇ 0 ԂB
     *
     * @else
     * @brief InPortPushConnector creation
     * @endif
     */
    InPortConnector*
    createConnector(ConnectorProfile& cprof, coil::Properties& prop,
                    InPortProvider* provider);
    /*!
     * @if jp
     * @brief InPortPullConnector ̐
     *
     * Connector 𐶐A m_connectors ɕۑB
     * Ɏsꍇ 0 ԂB
     *
     * @else
     * @brief InPortPullConnector creation
     * @endif
     */
    InPortConnector*
    createConnector(const ConnectorProfile& cprof, coil::Properties& prop,
                    OutPortConsumer* consumer);
  protected:
    bool m_singlebuffer;
    CdrBufferBase* m_thebuffer;
    coil::Properties m_properties;
    coil::vstring m_providerTypes;
    coil::vstring m_consumerTypes;
    ConnectorList m_connectors;
  };
}; // namespace RTC

#endif // RTC_INPORTBASE_H
