// -*- C++ -*-
/*!
 * @file  InPortConsumer.h
 * @brief InPortConsumer class
 * @date  $Date: 2007-12-31 03:08:03 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: InPortConsumer.h 1329 2009-05-18 00:54:12Z n-ando $
 *
 */

#ifndef InPortConsumer_h
#define InPortConsumer_h

#include <coil/Factory.h>
#include <rtm/DataPortStatus.h>

namespace coil
{
  class Properties;
};

namespace SDOPackage
{
  class NVList;
};

class cdrMemoryStream;

namespace RTC
{
  /*!
   * @if jp
   *
   * @class InPortConsumer
   *
   * @brief InPortConsumer ۃNX
   *
   * ̓|[gRV[}̂߂̒ۃC^[tF[XNX
   * eۃNX́Aȉ̏z֐̎񋟂Ȃ΂ȂȂB
   * - push(): f[^M
   * - clone(): |[g̃Rs[
   * - subscribeInterface(): f[^oʒmւ̓o^
   * - unsubscribeInterface(): f[^oʒm̓o^
   *
   * @since 0.4.0
   *
   * @else
   * @class InPortConsumer
   *
   * @brief InPortConsumer abstract class
   *
   * This is the abstract interface class for the input port Consumer.
   * Concrete classes must implement the following pure virtual functions.
   * - push(): Send data
   * - clone(): Copy ports
   * - subscribeInterface(): Subscribe the data send notification
   * - unsubscribeInterface(): Unsubscribe the data send notification
   *
   * @since 0.4.0
   *
   * @endif
   *
   */
  class InPortConsumer
    : public DataPortStatus
  {
  public:
    DATAPORTSTATUS_ENUM
    /*!
     * @if jp
     * @brief RXgN^
     *
     * RXgN^
     *
     * @else
     * @brief Constructor
     *
     * Constructor
     *
     * @endif
     */
    //InPortConsumer();
    
    /*!
     * @if jp
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~InPortConsumer(void){};

    /*!
     * @if jp
     * @brief ݒ菉
     *
     * InPortConsumer̊eݒs
     *
     * @else
     * @brief Initializing configuration
     *
     * This operation would be called to configure this consumer
     * in initialization.
     *
     * @endif
     */
    virtual void init(coil::Properties& prop) = 0;

    /*!
     * @if jp
     * @brief ڑւ̃f[^M
     *
     * ڑ̃|[gփf[^𑗐M邽߂̏z֐B
     *
     * @else
     * @brief Send data to the destination port
     *
     * Pure virtual function to send data to the destination port.
     *
     * @endif
     */
    virtual ReturnCode put(const cdrMemoryStream& data) = 0;

    /*!
     * @if jp
     * @brief InterfaceProfileJ
     *
     * InterfaceProfileJB
     * Ŏw肷vpeB NameValue IuWFNg
     * dataport.interface_type l𒲂ׁAY|[gɐݒ肳Ă
     * C^[tF[X^Cvƈvꍇ̂ݏ擾B
     *
     * @param properties InterfaceProfile󂯎vpeB
     *
     * @else
     * @brief Publish InterfaceProfile information
     *
     * Publish interfaceProfile information.
     * Check the dataport.interface_type value of the NameValue object 
     * specified by an argument in property information and get information
     * only when the interface type of the specified port is matched.
     *
     * @param properties Properties to get InterfaceProfile information
     *
     * @endif
     */
    virtual void publishInterfaceProfile(SDOPackage::NVList& properties) = 0;

    /*!
     * @if jp
     * @brief f[^oʒm󂯎ւ̓o^
     *
     * w肳ꂽvpeB̓eɊÂāAf[^oʒm̎󂯎ɓo^
     * ߂̏z֐B
     *
     * @param properties o^ɎQƂvpeB
     *
     * @return o^
     *
     * @else
     * @brief Subscribe the data send notification
     *
     * Pure virtual function to subscribe the data send notification
     * based on specified property information.
     *
     * @param properties Properties for reference when subscribing
     *
     * @return Subscription result
     *
     * @endif
     */
    virtual bool subscribeInterface(const SDOPackage::NVList& properties) = 0;
    
    /*!
     * @if jp
     * @brief f[^oʒm󂯎肩̓o^
     *
     * f[^oʒm̎󂯎肩o^邽߂̏z֐B
     *
     * @param properties o^ɎQƂvpeB
     *
     * @else
     * @brief Unsubscribe the data send notification
     *
     * Pure virtual function to unsubscribe the data send notification.
     *
     * @param properties Properties for reference when unsubscribing
     *
     * @endif
     */
    virtual void unsubscribeInterface(const SDOPackage::NVList& properties) = 0;
    
    /*!
     * @if jp
     * @brief C^[tF[Xvt@CJ邽̃t@N^
     * @else
     * @brief Functor to publish interface profile
     * @endif
     */
    struct publishInterfaceProfileFunc
    {
      publishInterfaceProfileFunc(SDOPackage::NVList& prop) : m_prop(prop) {}
      void operator()(InPortConsumer* consumer)
      {
        consumer->publishInterfaceProfile(m_prop);
      }
      SDOPackage::NVList& m_prop;
    };

    /*!
     * @if jp
     * @brief C^[tF[Xvt@CJ邽̃t@N^
     * @else
     * @brief Functor to publish interface profile
     * @endif
     */
    struct subscribeInterfaceFunc
    {
      subscribeInterfaceFunc(SDOPackage::NVList& prop) : m_prop(prop) {}
      bool operator()(InPortConsumer* consumer)
      {
        return consumer->subscribeInterface(m_prop);
      }
      SDOPackage::NVList& m_prop;
    };

  
  };

  typedef ::coil::GlobalFactory<InPortConsumer> InPortConsumerFactory;
};     // namespace RTC

template class ::coil::GlobalFactory<RTC::InPortConsumer>;

#endif // InPortConsumer_h
