// -*- C++ -*-
/*!
 * @file  InPortCorbaCdrConsumer.h
 * @brief InPortCorbaCdrConsumer class
 * @date  $Date: 2007-12-31 03:08:03 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: InPortCorbaCdrConsumer.h 1255 2009-04-07 01:09:47Z kurihara $
 *
 */

#ifndef RTC_INPORTCORBACDRCONSUMER_H
#define RTC_INPORTCORBACDRCONSUMER_H


//#include <rtm/BufferBase.h>

#include <rtm/idl/DataPortSkel.h>
#include <rtm/CorbaConsumer.h>
#include <rtm/InPortConsumer.h>
#include <rtm/Manager.h>

namespace RTC
{
  /*!
   * @if jp
   *
   * @class InPortCorbaCdrConsumer
   *
   * @brief InPortCorbaCdrConsumer NX
   *
   * ʐMi CORBA 𗘗p̓|[gRV[}̎NXB
   *
   * @param DataType {|[gɂĈf[^^
   *
   * @since 1.0
   *
   * @else
   * @class InPortCorbaCdrConsumer
   *
   * @brief InPortCorbaCdrConsumer class
   *
   * This is an implementation class of the input port Consumer 
   * that uses CORBA for means of communication.
   *
   * @param DataType Data type for this port
   *
   * @since 0.4.0
   *
   * @endif
   */
  class InPortCorbaCdrConsumer
    : public InPortConsumer,
      public CorbaConsumer< ::OpenRTM::InPortCdr >
  {
  public:
    DATAPORTSTATUS_ENUM
    /*!
     * @if jp
     * @brief RXgN^
     *
     * RXgN^
     *
     * @param buffer YRV[}Ɋ蓖Ăobt@IuWFNg
     *
     * @else
     * @brief Constructor
     *
     * Constructor
     *
     * @param buffer The buffer object that is attached to this Consumer
     *
     * @endif
     */
    InPortCorbaCdrConsumer(void);
    
    /*!
     * @if jp
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~InPortCorbaCdrConsumer(void);

    /*!
     * @if jp
     * @brief ݒ菉
     *
     * InPortConsumer̊eݒs
     *
     * @else
     * @brief Initializing configuration
     *
     * This operation would be called to configure this consumer
     * in initialization.
     *
     * @endif
     */
    virtual void init(coil::Properties& prop);

    /*!
     * @if jp
     * @brief ڑւ̃f[^M
     *
     * ڑ̃|[gփf[^𑗐M邽߂̏z֐B
     *
     * @else
     * @brief Send data to the destination port
     *
     * Pure virtual function to send data to the destination port.
     *
     * @endif
     */
    virtual ReturnCode put(const cdrMemoryStream& data);

    /*!
     * @if jp
     * @brief InterfaceProfileJ
     *
     * InterfaceProfileJB
     * Ŏw肷vpeB NameValue IuWFNg
     * dataport.interface_type l𒲂ׁAY|[gɐݒ肳Ă
     * C^[tF[X^Cvƈvꍇ̂ݏ擾B
     *
     * @param properties InterfaceProfile󂯎vpeB
     *
     * @else
     * @brief Publish InterfaceProfile information
     *
     * Publish interfaceProfile information.
     * Check the dataport.interface_type value of the NameValue object 
     * specified by an argument in property information and get information
     * only when the interface type of the specified port is matched.
     *
     * @param properties Properties to get InterfaceProfile information
     *
     * @endif
     */
    virtual void publishInterfaceProfile(SDOPackage::NVList& properties);

    /*!
     * @if jp
     * @brief f[^Mʒmւ̓o^
     *
     * w肳ꂽvpeBɊÂāAf[^oʒm̎󂯎ɓo^B
     *
     * @param properties o^
     *
     * @return o^(o^:trueAo^s:false)
     *
     * @else
     * @brief Subscribe to the data sending notification
     *
     * Subscribe to the data sending notification based on specified 
     * property information.
     *
     * @param properties Information for subscription
     *
     * @return Subscription result (Successful:true, Failed:false)
     *
     * @endif
     */
    virtual bool subscribeInterface(const SDOPackage::NVList& properties);
    
    /*!
     * @if jp
     * @brief f[^Mʒm̓o^
     *
     * f[^oʒm̎󂯎肩o^B
     *
     * @param properties o^
     *
     * @else
     * @brief Unsubscribe the data send notification
     *
     * Unsubscribe the data send notification.
     *
     * @param properties Information for unsubscription
     *
     * @endif
     */
    virtual void unsubscribeInterface(const SDOPackage::NVList& properties);

  private:
    /*!
     * @if jp
     * @brief IOR񂩂IuWFNgQƂ擾
     *
     * @return true: 擾, false: 擾s
     *
     * @else
     * @brief Getting object reference fromn IOR string
     *
     * @return true: succeeded, false: failed
     *
     * @endif
     */
    bool subscribeFromIor(const SDOPackage::NVList& properties);

    /*!
     * @if jp
     * @brief Any璼ڃIuWFNgQƂ擾
     *
     * @return true: 擾, false: 擾s
     *
     * @else
     * @brief Getting object reference fromn Any directry
     *
     * @return true: succeeded, false: failed
     *
     * @endif
     */
    bool subscribeFromRef(const SDOPackage::NVList& properties);

    /*!
     * @if jp
     * @brief ڑ(IOR)
     *
     * @return true: 擾, false: 擾s
     *
     * @else
     * @brief ubsubscribing (IOR version)
     *
     * @return true: succeeded, false: failed
     *
     * @endif
     */
    bool unsubscribeFromIor(const SDOPackage::NVList& properties);

    /*!
     * @if jp
     * @brief ڑ(Object reference)
     *
     * @return true: 擾, false: 擾s
     *
     * @else
     * @brief ubsubscribing (Object reference version)
     *
     * @return true: succeeded, false: failed
     *
     * @endif
     */
    bool unsubscribeFromRef(const SDOPackage::NVList& properties);
    
  private:
    mutable Logger rtclog;
    coil::Properties m_properties;
  };
};     // namespace RTC

extern "C"
{
  void InPortCorbaCdrConsumerInit(void);
};

#endif // RTC_INPORTCORBACDRCONSUMER_H

