// -*- C++ -*-
/*!
 * @file Manager.h
 * @brief RTComponent manager class
 * @date $Date: 2007-12-31 03:08:04 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: Manager.h 1327 2009-05-18 00:50:59Z n-ando $
 *
 */

#ifndef Manager_h
#define Manager_h

#include <rtm/RTC.h>

#include <iostream>
#include <string>
#include <vector>

#include <coil/Mutex.h>
#include <coil/Guard.h>
#include <coil/Task.h>

#include <rtm/Factory.h>
#include <rtm/ECFactory.h>
#include <rtm/ObjectManager.h>
#include <rtm/SystemLogger.h>

namespace RTM
{
  class ManagerServant;
}

namespace coil
{
  class Timer;
};

namespace RTC
{
//  class Properties;
  class CorbaNaming;
  class ModuleManager;
  class NamingManager;
  class Manager;
  class RTObject_impl;
  typedef RTObject_impl RtcBase;

  typedef void (*ModuleInitProc)(Manager* manager);  
  
  /*!
   * @if jp
   * @class Manager
   * @brief Manager NX
   *
   * R|[lgȂǊȅǗs}l[WNXB
   *
   * @since 0.2.0
   *
   * @else
   * @class Manager
   * @brief Manager class
   *
   * This is a manager class that manages various information
   * such as components.
   *
   * @since 0.2.0
   *
   * @endif
   */
  class Manager
  {
    typedef coil::Mutex Mutex;
    typedef coil::Guard<Mutex> Guard;
  protected:
    /*!
     * @if jp
     * @brief Protected RXgN^
     *
     * Protected RXgN^
     *
     * @else
     * @brief Protected Constructor
     *
     * Protected Constructor
     *
     * @endif
     */
    Manager();
    
    /*!
     * @if jp
     * @brief Protected Rs[RXgN^
     *
     * Protected Rs[RXgN^
     *
     * @param manager Rs[}l[WIuWFNg
     *
     * @else
     * @brief Protected Copy Constructor
     *
     * Protected Copy Constructor
     *
     * @param manager Manager object of copy source
     *
     * @endif
     */
    Manager(const Manager& manager);
    //      Manager& operator=(const Manager& manager){return manager;};
    
  public:
    /*!
     * @if jp
     * @brief }l[W̏
     *
     * }l[W static o֐B
     * }l[WR}hC^ďB
     * }l[Wgpꍇ́AK̏o֐ init() 
     * Ă΂Ȃ΂ȂȂB
     * }l[W̃CX^X擾@ƂāAinit(), instance() 
     * 2 static o֐pӂĂ邪Ainit()ł
     * sȂ߁AManager ̐Ԃ̈ԍŏɂinit()ĂԕKvB
     *
     * }l[W̏
     * - initManager: Aconfigt@C̓ǂݍ݁ATuVXe
     * - initLogger: Logger
     * - initORB: ORB 
     * - initNaming: NamingService 
     * - initExecutionContext: ExecutionContext factory 
     * - initTimer: Timer 
     *
     * @param argc R}hC̐
     * @param argv R}hC
     * 
     * @return Manager ̗B̃CX^X̎Q
     *
     * @else
     * @brief Initialize manager
     *
     * This is the static member function to initialize the Manager.
     * The Manager is initialized by given commandline arguments.
     * To use the manager, this initialization member function init() must be
     * called. The manager has two static functions to get the instance such as
     * init() and instance(). Since initializing process is only performed by
     * the init() function, the init() has to be called at the beginning of
     * the lifecycle of the Manager.
     *
     * *Initialization of manager
     * - initManager: Argument processing, reading config file,
     *                initialization of subsystem
     * - initLogger: Initialization of Logger
     * - initORB: Initialization of ORB
     * - initNaming: Initialization of NamingService
     * - initExecutionContext: Initialization of ExecutionContext factory
     * - initTimer: Initialization of Timer
     *
     * @param argc The number of command line arguments. 
     * @param argv The array of the command line arguments.
     *
     * @return Reference of the unique instance of Manager
     *
     * @endif
     */
    static Manager* init(int argc, char** argv);
    
    /*!
     * @if jp
     * @brief }l[W̃CX^X̎擾
     *
     * }l[W̃CX^X擾 static o֐B
     * ̊֐ĂԑOɁAK̏o֐ init() Ă΂Ă
     * KvB
     *
     * @return Manager ̗B̃CX^X̎Q
     * 
     * @else
     *
     * @brief Get instance of the manager
     *
     * This is the static member function to get the instance of the Manager.
     * Before calling this function, ensure that the initialization function
     * "init()" is called.
     *
     * @return The only instance reference of the manager
     *
     * @endif
     */ 
    static Manager& instance();
    
    //============================================================
    // Manager
    //============================================================
    
    /*!
     * @if jp
     * @brief }l[WI
     *
     * }l[W̏IsB
     *
     * @else
     * @brief Terminate manager
     *
     * Terminate manager's processing
     *
     * @endif
     */
    void terminate();
    
    /*!
     * @if jp
     * @brief }l[WEVbg_E
     *
     * }l[W̏IsB
     * ORBIAďIB
     *
     * @else
     * @brief Shutdown Manager
     *
     * Terminate manager's processing.
     * After terminating ORB, shutdown manager in sync.
     *
     * @endif
     */
    void shutdown();
    
    /*!
     * @if jp
     * @brief }l[WȊ҂킹
     *
     * 邽߁A}l[WȊ҂킹sB
     *
     * @else
     * @brief Wait for Manager's termination
     *
     * Wait for Manager's termination to synchronize.
     *
     * @endif
     */
    void join();
    
    /*!
     * @if jp
     * @brief Oobt@̎擾
     *
     * }l[Wɐݒ肵Oobt@擾B
     *
     * @return }l[Wɐݒ肵Oobt@
     *
     * @else
     * @brief Get the log buffer
     *
     * Get the log buffer that has been set to manager.
     *
     * @return Log buffer to set to manager
     *
     * @endif
     */
    LogStreamBuf& getLogStreamBuf() {return m_logStreamBuf;}
    std::string& getLogLevel() {return m_config["logger.log_level"];}
    
    /*!
     * @if jp
     * @brief }l[WRtBM[V̎擾
     *
     * }l[Wɐݒ肵RtBM[V擾B
     *
     * @return }l[W̃RtBM[V
     *
     * @else
     * @brief Get the manager configuration
     *
     * Get the manager configuration that has been set to manager.
     *
     * @return Manager's configuration
     *
     * @endif
     */
    coil::Properties& getConfig() { return m_config;}
    
    /*!
     * @if jp
     *
     * @brief vV[W̃Zbg
     *
     * ̃Iy[V̓[UsW[̏vV[W
     * ݒ肷BŐݒ肳ꂽvV[ẂA}l[WA
     * ANeBuꂽAK؂ȃ^C~OŎsB
     *
     * @param proc vV[W̊֐|C^
     *
     * @else
     *
     * @brief Set initial procedure
     *
     * This operation sets the initial procedure call to process module
     * initialization, other user defined initialization and so on.
     * The given procedure will be called at the proper timing after the 
     * manager initialization, activation and run.
     *
     * @param proc A function pointer to the initial procedure call
     *
     * @endif
     */
    void setModuleInitProc(ModuleInitProc proc);
    
    /*!
     * @if jp
     *
     * @brief Manager̃ANeBu
     *
     * ̃Iy[V͈ȉ̏s
     * - CORBA POAManager ̃ANeBu
     * - }l[WCORBAIuWFNg̃ANeBu
     * - Manager ̃IuWFNgQƂ̓o^
     *
     * ̃Iy[V́A}l[W̏ArunManager()
     * ̑OɌĂԕKvB
     *
     *
     * @return (ANeBu:trueAs:false)
     *
     * @else
     *
     * @brief Activate the Manager
     *
     * This operation do the following:
     * - Activate CORBA POAManager
     * - Activate Manager CORBA object
     * - Bind object reference of the Manager to the nameserver
     *
     * This operation should be invoked after Manager:init(),
     * and before runManager().
     *
     * @return Activation result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool activateManager();
    
    /*!
     * @if jp
     *
     * @brief Manager̎s
     *
     * ̃Iy[V̓}l[W̃C[vsB
     * ̃C[vł́ACORBA ORB̃Cxg[v
     * BftHgł́ÃIy[V̓ubNA
     * Manager::destroy() Ă΂܂ŏ߂ȂB
     *  no_block  true ɐݒ肳Ăꍇ́AŃCxg[v
     * XbhNAubNɏ߂B
     *
     * @param no_block false: ubLO[h, true: mubLO[h
     *
     * @else
     *
     * @brief Run the Manager
     *
     * This operation processes the main event loop of the Manager.
     * In this main loop, CORBA's ORB event loop or other processes
     * are performed. As the default behavior, this operation is going to
     * blocking mode and never returns until Manager::destroy() is called.
     * When the given argument "no_block" is set to "true", this operation
     * creates a thread to process the event loop internally, and it doesn't
     * block and returns.
     *
     * @param no_block false: Blocking mode, true: non-blocking mode.
     *
     * @endif
     */
    void runManager(bool no_block = false);
    
    //============================================================
    // Module management
    //============================================================
    /*!
     * @if jp
     * @brief [CORBA interface] W[̃[h
     *
     * w肵R|[lg̃W[[hƂƂɁA
     * w肵֐sB
     *
     * @param fname   W[t@C
     * @param initfunc ֐
     * 
     * @else
     *
     * @brief [CORBA interface] Load module
     *
     * Load specified module (shared library, DLL etc..),
     * and invoke initialize function.
     *
     * @param fname    The module file name
     * @param initfunc The initialize function name
     *
     * @endif
     */  
    void load(const char* fname, const char* initfunc);
    
    /*!
     * @if jp
     *
     * @brief W[̃A[h
     *
     * W[A[h
     *
     * @param fname W[̃t@C
     * 
     * @else
     *
     * @brief Unload module
     *
     * Unload module.
     *
     * @param fname The module file name
     *
     * @endif
     */ 
    void unload(const char* fname);
    
    /*!
     * @if jp
     *
     * @brief SW[̃A[h
     *
     * W[ׂăA[h
     *
     * @else
     *
     * @brief Unload all modules
     *
     * Unload all modules.
     *
     * @endif
     */ 
    void unloadAll();
    
    /*!
     * @if jp
     * @brief [hς݂̃W[Xg擾
     *
     * ݃}l[WɃ[hĂ郂W[̃Xg擾B
     *
     * @return [hς݃W[Xg
     *
     * @else
     * @brief Get a list of loaded modules
     *
     * Get module list that is currently loaded into manager.
     *
     * @return Module list that has been loaded.
     *
     * @endif
     */
    std::vector<coil::Properties> getLoadedModules();
    
    /*!
     * @if jp
     * @brief [h\ȃW[Xg擾
     *
     * [h\W[̃Xg擾B
     * (݂ModuleManagerŖ)
     *
     * @return [h\W[@Xg
     *
     * @else
     * @brief Get a list of loadable modules
     *
     * Get loadable module list.
     * (Currently, unimplemented on ModuleManager side)
     *
     * @return Loadable module list
     *
     * @endif
     */
  std::vector<coil::Properties> getLoadableModules();
    
    //============================================================
    // Component Factory Management
    //============================================================
    /*!
     * @if jp
     * @brief RTR|[lgpt@Ngo^
     *
     * RTR|[lg̃CX^X𐶐邽߂
     * Factoryo^B
     *
     * @param profile RTR|[lg vt@C
     * @param new_func RTR|[lgp֐
     * @param delete_func RTR|[lgjp֐
     *
     * @return o^(o^:trueAs:false)
     *
     * @else
     * @brief Register RT-Component Factory
     *
     * Register Factory to create RT-Component's instances.
     *
     * @param profile RT-Component profile
     * @param new_func RT-Component creation function
     * @param delete_func RT-Component destruction function
     *
     * @return Registration result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool registerFactory(coil::Properties& profile,
			 RtcNewFunc new_func,
			 RtcDeleteFunc delete_func);
    std::vector<coil::Properties> getFactoryProfiles();
    /*!
     * @if jp
     * @brief ExecutionContextpt@Ngo^
     *
     * ExecutionContext̃CX^X𐶐邽߂
     * Factoryo^B
     *
     * @param name ΏExecutionContext
     * @param new_func ExecutionContextp֐
     * @param delete_func ExecutionContextjp֐
     *
     * @return o^(o^:trueAs:false)
     *
     * @else
     * @brief Register ExecutionContext Factory
     *
     * Register Factory to create ExecutionContext's instances.
     *
     * @param name ExecutionContext name for the creation 
     * @param new_func ExecutionContext creation function
     * @param delete_func ExecutionContext destruction function
     *
     * @return Registration result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool registerECFactory(const char* name,
			   ECNewFunc new_func,
			   ECDeleteFunc delete_func);
    
    /*!
     * @if jp
     * @brief t@NgSXg擾
     *
     * o^Ăt@Ng̑SXg擾B
     *
     * @return o^t@Ng Xg
     *
     * @else
     * @brief Get the list of all Factories
     *
     * Get the list of all factories that have been registered.
     *
     * @return Registered factory list
     *
     * @endif
     */
    std::vector<std::string> getModulesFactories();
    
    //============================================================
    // Component management
    //============================================================
    /*!
     * @if jp
     * @brief RTR|[lg𐶐
     *
     * w肵RTR|[lg̃CX^Xo^ꂽFactoryoR
     * ŐB
     *
     * R|[lg̊evt@C͈ȉ̗D揇ʂ
     * ݒ肳B
     *
     * -# createComponent() ̈ŗ^ꂽvt@C
     * -# rtc.confŎw肳ꂽOt@Cŗ^ꂽvt@C
     * --# category.instance_name.config_file
     * --# category.component_type.config_file
     * -# R[hɖߍ܂ꂽvt@C 
     *
     * CX^XꍇAĈȉ̏sB
     *  - Ot@CŐݒ肵RtBM[V̓ǂݍ݁Cݒ
     *  - ExecutionContext̃oChCJn
     *  - l[~OT[rXւ̓o^
     *
     * @param comp_args ΏRTR|[lgIDуRtBM[
     * VBtH[}bg͑傫 "id"  "configuration" 
     * ݂B
     *
     * comp_args:     [id]?[configuration]
     *                id ͕K{Aconfiguration̓IvV
     * id:            RTC:[vendor]:[category]:[implementation_id]:[version]
     *                RTC ͌Œ肩K{
     *                vendor, category, version ̓IvV
     *                implementation_id ͕K{
     *                IvVȗꍇł ":" ͏ȗs
     * configuration: [key0]=[value0]&[key1]=[value1]&[key2]=[value2].....
     *                RTCProperties̒lׂď㏑邱ƂłB
     *                key=value ̌`ŋLqA"&" ŋ؂
     *
     * Ⴆ΁A
     * RTC:jp.go.aist:example:ConfigSample:1.0?conf.default.str_param0=munya
     * RTC::example:ConfigSample:?conf.default.int_param0=100
     *
     * @return RTR|[lg̃CX^X
     *
     * @else
     * @brief Create RT-Components
     *
     * Create specified RT-Component's instances via registered Factory.
     * When its instances have been created successfully, the following
     * processings are also executed.
     *  - Read and set configuration information that was set by external file.
     *  - Bind ExecutionContext and start operation.
     *  - Register to naming service.
     *
     * @param module_name Target RT-Component names for the creation
     *
     * @return Created RT-Component's instances
     *
     * @endif
     */
    RTObject_impl* createComponent(const char* comp_args);
    ExecutionContextBase* createContext(const char* ec_args);
    
    /*!
     * @if jp
     * @brief RTR|[lg̓o^
     *
     * w肵RTR|[lg̃CX^Xl[~OT[rX
     * o^B
     *
     * @param comp o^ΏRTR|[lg
     *
     * @else
     * @brief Unregister RT-Components
     *
     * Unregister specified RT-Component's instances from naming service.
     *
     * @param comp Target RT-Components for the unregistration
     *
     * @endif
     */
    void cleanupComponent(RTObject_impl* comp);
    
    /*!
     * @if jp
     * @brief RTR|[lg𒼐 Manager ɓo^
     *
     * w肵RTR|[lg̃CX^X
     * t@NgoRł͂Ȃڃ}l[Wɓo^B
     *
     * @param comp o^ΏRTR|[lg̃CX^X
     *
     * @return o^(o^:trueAs:false)
     *
     * @else
     * @brief Register RT-Component directly without Factory
     *
     * Register specified RT-Component's instances not via Factory
     * to Manager directly.
     *
     * @param comp Target RT-Component's instances for the registration
     *
     * @return Registration result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool registerComponent(RTObject_impl* comp);
    
    /*!
     * @if jp
     * @brief RTR|[lg̓o^
     *
     * w肵RTR|[lg̓o^B
     *
     * @param comp o^ΏRTR|[lg̃CX^X
     *
     * @return o^(:trueAs:false)
     *
     * @else
     * @brief Unregister RT-Components
     *
     * Unregister specified RT-Components
     *
     * @param comp Target RT-Component's instances for the unregistration
     *
     * @return Unregistration result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool unregisterComponent(RTObject_impl* comp);
    
    
    /*!
     * @if jp
     * @brief Manager ɓo^ĂRTR|[lg폜
     *
     * }l[Wɓo^ĂRTR|[lg폜B
     * w肳ꂽRTR|[lgl[~OT[rX폜A
     * RTR|[lĝIƂƂɁACX^XB
     * (ł͖)
     *
     * @param instance_name 폜ΏRTR|[lg̃CX^X
     *
     * @else
     * @brief Unregister RT-Components that have been registered to Manager
     *
     * Unregister RT-Components that have been registered to manager
     * Remove specified RT-Component from naming service, terminate itself
     * and release its instances.
     * (Currently, not implemented)
     *
     * @param instance_name Target RT-Component's instances for the 
     *                      unregistration
     *
     * @endif
     */
    void deleteComponent(const char* instance_name);
//    void deleteComponent(RTObject_impl* comp);
    
    /*!
     * @if jp
     * @brief Manager ɓo^ĂRTR|[lg
     *
     * Manager ɓo^ĂRTR|[lgw肵̂ŌA
     * vR|[lg擾B
     *
     * @param instance_name ΏRTR|[lg̖
     *
     * @return ̂vRTR|[lg̃CX^X
     *
     * @else
     * @brief Get RT-Component's pointer
     *
     * Search RT-Component that has been registered to Manager by its specified
     * name, and get it that matches.
     *
     * @param instance_name Target RT-Component's name for searching
     *
     * @return Target RT-Component's instances that matches
     *
     * @endif
     */
    RTObject_impl* getComponent(const char* instance_name);
    
    /*!
     * @if jp
     * @brief Manager ɓo^ĂSRTR|[lg擾
     *
     * Manager ɓo^ĂRTR|[lg̑SCX^X擾B
     *
     * @return SRTR|[lg̃CX^XXg
     *
     * @else
     * @brief Get all RT-Components registered in the Manager
     *
     * Get all RT-Component's instances that have been registered to Manager.
     *
     * @return List of all RT-Component's instances
     *
     * @endif
     */
    std::vector<RTObject_impl*> getComponents();
    
    //============================================================
    // CORBA ֘A
    //============================================================
    /*!
     * @if jp
     * @brief ORB ̃|C^擾
     *
     * Manager ɐݒ肳ꂽ ORB ̃|C^擾B
     *
     * @return ORB IuWFNg
     *
     * @else
     * @brief Get the pointer to ORB
     *
     * Get the pointer to ORB that has been set to Manager.
     *
     * @return ORB object
     *
     * @endif
     */
    CORBA::ORB_ptr getORB();
    
    /*!
     * @if jp
     * @brief Manager  RootPOA ̃|C^擾
     *
     * Manager ɐݒ肳ꂽ RootPOA ւ̃|C^擾B
     *
     * @return RootPOAIuWFNg
     *
     * @else
     * @brief Get a pointer to RootPOA held by Manager
     *
     * Get the pointer to RootPOA that has been set to Manager.
     *
     * @return RootPOA object
     *
     * @endif
     */
    PortableServer::POA_ptr getPOA();
    
    /*!
     * @if jp
     * @brief Manager  POAManager 擾
     *
     * Manager ɐݒ肳ꂽ POAMAnager 擾B
     *
     * @return POA}l[W
     *
     * @else
     * @brief Get POAManager that Manager has
     *
     * Get POAMAnager that has been set to Manager.
     *
     * @return POA manager
     *
     * @endif
     */
    PortableServer::POAManager_ptr getPOAManager();
    
    //============================================================
    // Protected functions
    //============================================================
  protected:
    
    //============================================================
    // Manager initialize and finalization
    //============================================================
    /*!
     * @if jp
     * @brief Manager ̓
     * 
     * Manager ̓sB
     *  - Manager RtBM[V̐ݒ
     *  - Oo̓t@C̐ݒ
     *  - IpXbh̐
     *  - ^C}pXbh̐(^C}gp)
     *
     * @param argc R}hC̐
     * @param argv R}hC
     * 
     * @else
     * @brief Manager internal initialization
     * 
     * Execute Manager's internal initialization processing.
     *  - Set Manager configuration
     *  - Set log output file
     *  - Create termination processing thread
     *  - Create timer thread (when using timer)
     *
     * @param argc Number of commandline arguments
     * @param argv Commandline arguments
     * 
     * @endif
     */
    void initManager(int argc, char** argv);
    
    /*!
     * @if jp
     * @brief Manager ̏I
     *
     * Manager I
     * (C݂͖)
     *
     * @else
     * @brief Shutdown Manager
     *
     * Shutdown Manager
     * (However, not implemented now)
     *
     * @endif
     */
    void shutdownManager();
    
    //============================================================
    // Logger initialize and terminator
    //============================================================
    /*!
     * @if jp
     * @brief System logger ̏
     *
     * System logger ̏sB
     * RtBM[Vt@Cɐݒ肳ꂽɊÂA
     * K[̏CݒsB
     *
     * @return s(:trueAs:false)
     *
     * @else
     * @brief System logger initialization
     *
     * Initialize System logger.
     * Initialize logger and set it according to the set information in
     * configuration file,
     *
     * @return Initialization result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool initLogger();
    
    /*!
     * @if jp
     * @brief System Logger ̏I
     *
     * System LoggeȑIsB
     * obt@ɕێĂ郍O񂪑݂ꍇɂ́A
     * IɃOt@Cɏo͂AOt@CB
     *
     * @else
     * @brief System Logger finalization
     *
     * Finalize System Logger.
     * If log information stored in the buffer exists, output information
     * to the log file forcibly and close it.
     *
     * @endif
     */
    void shutdownLogger();
    
    //============================================================
    // ORB initialization and finalization
    //============================================================
    /*!
     * @if jp
     * @brief CORBA ORB ̏
     *
     * ɂ^ꂽݒORBB
     *
     * @return ORB (:trueAs:false)
     *
     * @else
     * @brief CORBA ORB initialization
     *
     * Initialize ORB based on the configuration given by arguments.
     *
     * @return ORB initialization result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool initORB();
    
    /*!
     * @if jp
     * @brief ORB ̃R}hCIvV쐬
     *
     * RtBM[Vɐݒ肳ꂽe
     * ORB ̋NIvV쐬B
     *
     * @return ORB NIvV
     *
     * @else
     * @brief Create ORB command options
     *
     * Create ORB launch options from configuration information
     * that has been set.
     *
     * @return ORB launch options
     *
     * @endif
     */
    std::string createORBOptions();
    
    /*!
     * @if jp
     * @brief ORB ̏I
     *
     * ORB ̏IsB
     * s҂݂̏ꍇɂ́ȀI܂ő҂B
     * ۂ̏Ił́APOA Manager񊈐A ORB ̃Vbg_Es
     * B
     *
     * @else
     * @brief ORB finalization
     *
     * Finalize ORB .
     * When the waiting process exists, wait until it completes.
     * In actual finalization, deactivate POA Manager and then shutdown of ORB.
     *
     * @endif
     */
    void shutdownORB();
    
    //============================================================
    // NamingService initialization and finalization
    //============================================================
    /*!
     * @if jp
     * @brief NamingManager ̏
     *
     * NamingManager ̏sB
     * ANamingManager gpȂ悤ɃvpeBɐݒ肳Ă
     * ꍇɂ͉ȂB
     * NamingManager gpꍇAvpeBɐݒ肳Ă
     * ftHg NamingServer o^B
     * ܂AIɏXV悤ɐݒ肳Ăꍇɂ́Aw肳ꂽ
     * ŎXVs߂̃^C}NƂƂɁAXVp\bh^C}
     * o^B
     *
     * @return (:trueAs:false)
     *
     * @else
     * @brief NamingManager initialization
     *
     * Initialize NamingManager .
     * However, operate nothing, if it is set to property that NamingManager
     * is not used.
     * Register default NamingServer that is set to property information,
     * when NamingManager is used.
     * Also, launch a timer that updates information automatically at specified
     * cycle and register the method for the update to the timer, when it is set
     * to update it reguraly.
     *
     * @return Initialization result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool initNaming();
    
    /*!
     * @if jp
     * @brief NamingManager ̏I
     *
     * NamingManager IB
     * o^ĂSvfAoChAIB
     *
     * @else
     * @brief NamingManager finalization
     *
     * Finalize NamingManager.
     * Unbind all registered elements and shutdown them.
     *
     * @endif
     */
    void shutdownNaming();
    
    //============================================================
    // Component management
    //============================================================
    /*!
     * @if jp
     * @brief NamingManager ɓo^Ă RTR|[lg̏I
     *
     * NamingManager ɓo^ĂRTR|[lg̃Xg擾A
     * SR|[lgIB
     *
     * @else
     * @brief NamingManager finalization
     *
     * Get a list of RT-Components that have been registered to NamingManager,
     * and shutdown all components.
     *
     * @endif
     */
    void shutdownComponents();


    /*!
     * @if jp
     * @brief 񂩂R|[lg^EvpeB𒊏o
     *
     * 񂩂R|[lg^ƃR|[lg̃vpeB𒊏oB
     * ^镶̃tH[}bg
     * [RTC type]?[key(0)]=[val(0)]&[key(1)]=[val(1)]...[key(n)]=[val(n)]
     * 
     * @return comp_arg ɃR|[lg^܂܂ĂȂꍇfalse
     * @param comp_arg  ׂ
     * @param comp_type oꂽR|[lǧ^
     * @param comp_prop oꂽR|[lg̃vpeB
     *
     * @else
     * @brief Extracting component type/properties from the given string
     *
     * This operation extracts component type name and its properties
     * from the figen character string.
     * The given string formats is the following.
     * [RTC type]?[key(0)]=[val(0)]&[key(1)]=[val(1)]...[key(n)]=[val(n)]
     * 
     * @return comp_arg false will returned if no component type in arg
     * @param comp_arg  character string to be processed
     * @param comp_type extracted component type name
     * @param comp_prop extracted component's properties
     *
     * @endif
     */
    bool procComponentArgs(const char* comp_arg,
                           coil::Properties& comp_id,
                           coil::Properties& comp_conf);
    bool procContextArgs(const char* ec_args,
                         std::string& ec_id,
                         coil::Properties& ec_conf);

    /*!
     * @if jp
     * @brief RTR|[lg̃RtBM[V
     *
     * RTR|[lǧ^уCX^XɋLڂꂽvpeBt@C
     * ǂݍ݁AR|[lgɐݒ肷B
     * ܂AeR|[lg NamingService o^̖̂擾Aݒ肷B
     *
     * @param comp RtBM[VΏRTR|[lg
     *
     * @else
     * @brief Configure RT-Component
     *
     * Read property files described each RT-Component's type and instance,
     * and configure it to the component.
     * Also, get each component's registered name when registering to
     * NamingService and configure it.
     *
     * @param comp Target RT-Component for the configuration
     *
     * @endif
     */
    void configureComponent(RTObject_impl* comp, const coil::Properties& prop);
    
    /*!
     * @if jp
     * @brief ExecutionContextManager ̏
     *
     * gpe ExecutionContext ̏sAe ExecutionContext 
     * p Factory  ExecutionContextManager ɓo^B
     *
     * @return ExecutionContextManager s
     *         (:trueAs:false)
     *
     * @else
     * @brief ExecutionContextManager initialization
     *
     * Initialize each ExecutionContext that is used, and register each 
     * ExecutionContext creation Factory to ExecutionContextManager.
     *
     * @return ExecutionContextManager initialization result
     *          (Successful:true, Failed:false)
     *
     * @endif
     */
    bool initExecContext();
    bool initComposite();
    bool initFactories();

    /*!
     * @if jp
     * @brief Timer ̏
     *
     * gpe Timer ̏sB
     * (̎ł͉Ȃ)
     *
     * @return Timer s(:trueAs:false)
     *
     * @else
     * @brief Timer initialization
     *
     * Initialize each Timer that is used.
     * (In current implementation, nothing is done.)
     *
     * @return Timer Initialization result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool initTimer();

    bool initManagerServant();
    RTM::ManagerServant* m_mgrservant;

    /*!
     * @if jp
     * @brief vpeB̃}[W
     *
     * w肳ꂽt@Cɐݒ肳ĂvpeB[hA
     * ̐ݒς݃vpeBƃ}[WB
     *
     * @param prop }[WΏۃvpeB
     * @param file_name vpeB񂪋LqĂt@C
     *
     * @return }[Ws(}[W:trueA}[Ws:false)
     *
     * @else
     * @brief Merge property information
     *
     * Load property information that is configured in the specified file,
     * and merge existing properties that has been configured.
     *
     * @param prop Target properties for the merge
     * @param file_name File name that property information is described
     *
     * @return Merge result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool mergeProperty(coil::Properties& prop, const char* file_name);
    
    /*!
     * @if jp
     * @brief NamingServer ɓo^ۂ̓o^gݗĂ
     *
     * w肳ꂽƃvpeB NameServer ɓo^ۂ̏
     * gݗĂB
     * ewp̈Ӗ͈ȉ̂Ƃ
     * - % : ReLXg̋؂
     * - n : CX^X
     * - t : ^
     * - m : ^
     * - v : o[W
     * - V : x_[
     * - c : JeS
     * - h : zXg
     * - M : }l[W
     * - p : vZXID
     *
     * @param naming_format NamingService o^񏑎w
     * @param prop gpvpeB
     *
     * @return w菑ϊ
     *
     * @else
     * @brief Construct registration information when registering to 
     *        Naming server
     *
     * Construct information when registering to NameServer based on specified
     * format and property information.
     * Each format specification character means as follows:
     * - % : Break of Context
     * - n : Instance's name
     * - t : Type name
     * - m : Type name
     * - v : Version
     * - V : Vender
     * - c : Category
     * - h : Host name
     * - M : Manager name
     * - p : Process ID
     *
     * @param naming_format Format specification for NamingService registration
     * @param prop Property information that is used
     *
     * @return Specification format conversion result
     *
     * @endif
     */
    std::string formatString(const char* naming_format,
			     coil::Properties& prop);
    
    //============================================================
    // protected ϐ
    //============================================================
    
    //------------------------------------------------------------
    // static var
    //------------------------------------------------------------
    /*!
     * @if jp
     * @brief B Manager ւ̃|C^
     * @else
     * @brief The pointer to the Manager
     * @endif
     */
    static Manager* manager;
    
    /*!
     * @if jp
     * @brief B Manager ւ̃|C^ɑ΂ mutex
     * @else
     * @brief The mutex of the pointer to the Manager 
     * @endif
     */
    static Mutex mutex;
    
    //------------------------------------------------------------
    // CORBA var
    //------------------------------------------------------------
    /*!
     * @if jp
     * @brief ORB ւ̃|C^
     * @else
     * @brief The pointer to the ORB
     * @endif
     */
    CORBA::ORB_var m_pORB;
    
    /*!
     * @if jp
     * @brief POA ւ̃|C^
     * @else
     * @brief The pointer to the POA
     * @endif
     */
    PortableServer::POA_var m_pPOA;
    
    /*!
     * @if jp
     * @brief POAManager ւ̃|C^
     * @else
     * @brief The pointer to the POAManager
     * @endif
     */
    PortableServer::POAManager_var m_pPOAManager;
    
    //------------------------------------------------------------
    // Manager's variable
    //------------------------------------------------------------
    /*!
     * @if jp
     * @brief [U֐ւ̃|C^
     * @else
     * @brief User's initialization function's pointer
     * @endif
     */
    ModuleInitProc m_initProc;
    
    /*!
     * @if jp
     * @brief Manager  configuration i[ Properties
     * @else
     * @brief Managaer's configuration Properties
     * @endif
     */
    coil::Properties m_config;
    
    /*!
     * @if jp
     * @brief ModuleManager ւ̃|C^
     * @else
     * @brief The pointer to the ModuleManager
     * @endif
     */
    ModuleManager* m_module;
    
    /*!
     * @if jp
     * @brief NamingManager ւ̃|C^
     * @else
     * @brief The pointer to the NamingManager
     * @endif
     */
    NamingManager* m_namingManager;
    
    /*!
     * @if jp
     * @brief Timer Object
     * @else
     * @brief Timer Object
     * @endif
     */
    coil::Timer* m_timer;
    
    //------------------------------------------------------------
    // Logger
    //------------------------------------------------------------
    /*!
     * @if jp
     * @brief K[obt@
     * @else
     * @brief Logger buffer
     * @endif
     */
    LogStreamBuf m_logStreamBuf;
    
    /*!
     * @if jp
     * @brief K[Xg[
     * @else
     * @brief Logger stream
     * @endif
     */
    Logger rtclog;

    std::vector<std::ofstream*> m_logfiles;
    
    //============================================================
    // R|[lg}l[W
    //============================================================
    // ObjectManager ֓nqNX
    struct InstanceName
    {
      InstanceName(RTObject_impl* comp);
      InstanceName(const char* name);
      InstanceName(const std::string name);
      bool operator()(RTObject_impl* comp);
      std::string m_name;
    };
    
    typedef ObjectManager<std::string, RTObject_impl, InstanceName> ComponentManager;

    /*!
     * @if jp
     * @brief R|[lg}l[W
     * @else
     * @brief ComponentManager
     * @endif
     */
    ComponentManager m_compManager;
    
    //============================================================
    // R|[lgt@Ng
    //============================================================
    // R|[lgt@Ng֓nqNX
    class FactoryPredicate
    {
    public:
      FactoryPredicate(const char* imple_id)
        : m_vendor(""), m_category(""), m_impleid(imple_id), m_version("")
      {
      }
      FactoryPredicate(const coil::Properties& prop)
        : m_vendor(prop["vendor"]),
          m_category(prop["category"]),
          m_impleid(prop["implementation_id"]),
          m_version(prop["version"])
      {
      }
      FactoryPredicate(FactoryBase* factory)
	: m_vendor(factory->profile()["vendor"]),
          m_category(factory->profile()["category"]),
          m_impleid(factory->profile()["implementation_id"]),
          m_version(factory->profile()["version"])
      {
      }
      bool operator()(FactoryBase* factory)
      {
        // implementation_id must not be empty
        if (m_impleid.empty()) return false;

        const coil::Properties& prop(factory->profile());

        if (m_impleid != prop["implementation_id"])
          return false;
        if (!m_vendor.empty()   && m_vendor != prop["vendor"])
          return false;
        if (!m_category.empty() && m_category != prop["category"])
          return false;
        if (!m_version.empty()  && m_version != prop["version"])
          return false;
          
        return true;
      }
    private:
      std::string m_vendor;
      std::string m_category;
      std::string m_impleid;
      std::string m_version;
    };
    
    /*!
     * @if jp
     * @brief R|[lgt@Ng
     * @else
     * @brief ComponentFactory
     * @endif
     */
    typedef ObjectManager<const coil::Properties, FactoryBase,
			  FactoryPredicate> FactoryManager;

    /*!
     * @if jp
     * @brief ComponentManager
     * @else
     * @brief ComponentManager
     * @endif
     */
    FactoryManager m_factory;
    
    //============================================================
    // ExecutionContextt@Ng
    //============================================================
    // ECt@Ng֓nqNX
    struct ECFactoryPredicate
    {
      ECFactoryPredicate(const char* name) : m_name(name){};
      ECFactoryPredicate(ECFactoryBase* factory)
	: m_name(factory->name()) {};
      bool operator()(ECFactoryBase* factory)
      {
	return m_name == factory->name();
      }
      std::string m_name;
    };
    typedef ObjectManager<const char*,
			  ECFactoryBase,
			  ECFactoryPredicate> ECFactoryManager;
    
    /*!
     * @if jp
     * @brief ExecutionContext }l[W
     * @else
     * @brief ExecutionContext Manager
     * @endif
     */
    ECFactoryManager m_ecfactory;
    
    /*!
     * @if jp
     * @brief ExecutionContext Xg
     * @else
     * @brief ExecutionContext list
     * @endif
     */
    std::vector<ExecutionContextBase*> m_ecs;
    
    // t@NgXgAbv邽߂̃t@N^
    struct ModuleFactories
    {
      void operator()(FactoryBase* f)
      {
	modlist.push_back(f->profile().getProperty("implementation_id"));
      }
      std::vector<std::string> modlist;
    };
    
    //------------------------------------------------------------
    // ORB runner
    //------------------------------------------------------------
    /*!
     * @if jp
     * @class OrbRunner
     * @brief OrbRunner NX
     *
     * ORB spwp[NXB
     *
     * @since 0.4.0
     *
     * @else
     * @class OrbRunner
     * @brief OrbRunner class
     *
     * ORB exrcution helper class
     *
     * @since 0.4.0
     *
     * @endif
     */
    class OrbRunner
      : public coil::Task
    {
    public:
      /*!
       * @if jp
       * @brief RXgN^
       *
       * RXgN^
       *
       * @else
       * @brief Constructor
       *
       * Constructor
       *
       * @endif
       */
      OrbRunner(CORBA::ORB_ptr orb) : m_pORB(orb)
      {
	open(0);
      };
      
      /*!
       * @if jp
       * @brief ORB 
       *
       * ORB 
       *
       * @param args 
       *
       * @return 
       *
       * @else
       * @brief ORB activation processing
       *
       * ORB activation processing.
       *
       * @param args ORB activation processing
       *
       * @return Activation result
       *
       * @endif
       */
      virtual int open(void *args)
      {
	activate();
	return 0;
      }
      
      /*!
       * @if jp
       * @brief ORB Jn
       *
       * ORB Jn
       *
       * @return Jn
       *
       * @else
       * @brief ORB start processing
       *
       * ORB start processing
       *
       * @return Starting result
       *
       * @endif
       */
      virtual int svc(void)
      {
	m_pORB->run();
//	Manager::instance().shutdown();
	return 0;
      }
      
      /*!
       * @if jp
       * @brief ORB I
       *
       * ORB I
       *
       * @param flags ItO
       *
       * @return I
       *
       * @else
       * @brief ORB close processing
       *
       * ORB close processing.
       *
       * @param flags Flag of close processing
       *
       * @return Close result
       *
       * @endif
       */
      virtual int close(unsigned long flags)
      {
	return 0;
      }
    private:
      CORBA::ORB_ptr m_pORB;
    };
    OrbRunner* m_runner;
    
    //------------------------------------------------------------
    // Manager Terminator
    //------------------------------------------------------------
    /*!
     * @if jp
     * @class Terminator
     * @brief Terminator NX
     *
     * ORB Ipwp[NXB
     *
     * @since 0.4.0
     *
     * @else
     * @class Terminator
     * @brief Terminator class
     *
     * ORB termination helper class.
     *
     * @since 0.4.0
     *
     * @endif
     */
    class Terminator
      : public coil::Task
    {
    public:
      /*!
       * @if jp
       * @brief RXgN^
       *
       * RXgN^
       *
       * @param manager }l[WEIuWFNg
       *
       * @else
       * @brief Constructor
       *
       * Constructor
       *
       * @param manager Manager object
       *
       * @endif
       */
      Terminator(Manager* manager) : m_manager(manager) {};
      
      /*!
       * @if jp
       * @brief I
       *
       * ORBC}l[WIJnB
       *
       * @else
       * @brief Termination processing
       *
       * Start ORB and manager's termination processing.
       *
       * @endif
       */
      void terminate()
      {
	open(0);
      }
      
      /*!
       * @if jp
       * @brief I
       *
       * I
       *
       * @param args 
       *
       * @return 
       *
       * @else
       * @brief Termination processing activation
       *
       * Termination processing activation.
       *
       * @param args Activation argument
       *
       * @return Activation result
       *
       * @endif
       */
      virtual int open(void *args)
      {
	activate();
	return 0;
      }
      
      /*!
       * @if jp
       * @brief ORBC}l[WI
       *
       * ORBC}l[WI
       *
       * @return I
       *
       * @else
       * @brief ORB and manager's termination processing
       *
       * ORB and manager's termination processing.
       *
       * @return Termination result
       *
       * @endif
       */
      virtual int svc(void)
      {
	Manager::instance().shutdown();
	return 0;
      }
      Manager* m_manager;
    };
    
    /*!
     * @if jp
     * @brief Terminator
     * @else
     * @brief Terminator
     * @endif
     */
    Terminator* m_terminator;
    
    struct Term
    {
      int waiting;
      Mutex mutex;
    };
    Term m_terminate;
  }; // class Manager
}; // namespace RTC

#endif // Manager_h
