// -*- C++ -*-
/*!
 * @file  OutPortCorbaCdrConsumer.h
 * @brief OutPortCorbaCdrConsumer class
 * @date  $Date: 2008-01-13 10:28:27 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2009
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: OutPortCorbaCdrConsumer.h 1254 2009-04-07 01:09:35Z kurihara $
 *
 */

#ifndef RTC_OUTPORTCORBACDRCONSUMER_H
#define RTC_OUTPORTCORBACDRCONSUMER_H

#include <rtm/idl/DataPortSkel.h>
#include <rtm/CorbaConsumer.h>
#include <rtm/OutPortConsumer.h>

namespace RTC
{
  // forward decl

  /*!
   * @if jp
   * @class OutPortCorbaCdrConsumer
   *
   * @brief OutPortCorbaCdrConsumer NX
   *
   * ʐMi CORBA 𗘗po̓|[gRV[}̎NXB
   *
   * @param DataType {|[gɂĈf[^^
   *
   * @since 0.4.0
   *
   * @else
   * @class OutPortCorbaCdrConsumer
   *
   * @brief OutPortCorbaCdrConsumer class
   *
   * This is an implementation class of the output Consumer 
   * that uses CORBA for means of communication.
   *
   * @param DataType Data type for this port
   *
   * @since 0.4.0
   *
   * @endif
   */
  class OutPortCorbaCdrConsumer
    : public OutPortConsumer,
      public CorbaConsumer< ::OpenRTM::OutPortCdr >
  {
  public:
    DATAPORTSTATUS_ENUM

    /*!
     * @if jp
     * @brief RXgN^
     *
     * RXgN^
     *
     * @param buffer {|[gɊ蓖Ăobt@
     *
     * @else
     * @brief Constructor
     *
     * Constructor
     *
     * @param buffer Buffer that is attached to this port
     *
     * @endif
     */
    OutPortCorbaCdrConsumer();

    /*!
     * @if jp
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~OutPortCorbaCdrConsumer(void); 

    /*!
     * @if jp
     * @brief ݒ菉
     *
     * OutPortConsumer̊eݒsBNXł́A^ꂽ
     * PropertiesKvȏ擾ĊeݒsB init() 
     * ́AOutPortProviderエсAڑɂꂼĂ΂
     * \BāÅ֐͕Ă΂邱Ƃz肵ċL
     * qׂłB
     * 
     * @param prop ݒ
     *
     * @else
     *
     * @brief Initializing configuration
     *
     * This operation would be called to configure in initialization.
     * In the concrete class, configuration should be performed
     * getting appropriate information from the given Properties data.
     * This function might be called right after instantiation and
     * connection sequence respectivly.  Therefore, this function
     * should be implemented assuming multiple call.
     *
     * @param prop Configuration information
     *
     * @endif
     */
    virtual void init(coil::Properties& prop);

    /*!
     * @if jp
     * @brief obt@Zbg
     *
     * OutPortConsumerf[^oobt@ZbgB
     * łɃZbgꂽobt@ꍇAȑÕobt@ւ
     * |C^ɑ΂ď㏑B
     * OutPortProvider̓obt@̏L肵ĂȂ̂ŁA
     * obt@̍폜̓[U̐ӔCōsȂ΂ȂȂB
     *
     * @param buffer OutPortProviderf[^oobt@ւ̃|C^
     *
     * @else
     * @brief Setting outside buffer's pointer
     *
     * A pointer to a buffer from which OutPortProvider retrieve data.
     * If already buffer is set, previous buffer's pointer will be
     * overwritten by the given pointer to a buffer.  Since
     * OutPortProvider does not assume ownership of the buffer
     * pointer, destructor of the buffer should be done by user.
     * 
     * @param buffer A pointer to a data buffer to be used by OutPortProvider
     *
     * @endif
     */
    virtual void setBuffer(CdrBufferBase* buffer);

    /*!
     * @if jp
     * @brief f[^ǂݏo
     *
     * ݒ肳ꂽf[^ǂݏoB
     *
     * @param data ǂݏof[^󂯎IuWFNg
     *
     * @return f[^ǂݏo(ǂݏo:trueAǂݏos:false)
     *
     * @else
     * @brief Read data
     *
     * Read set data
     *
     * @param data Object to receive the read data
     *
     * @return Read result (Successful:true, Failed:false)
     *
     * @endif
     */
    virtual ReturnCode get(cdrMemoryStream& data);

    /*!
     * @if jp
     * @brief f[^Mʒmւ̓o^
     *
     * w肳ꂽvpeBɊÂāAf[^Mʒm̎󂯎ɓo^B
     *
     * @param properties o^
     *
     * @return o^(o^:trueAo^s:false)
     *
     * @else
     * @brief Subscribe the data receive notification
     *
     * Subscribe the data receive notification based on specified property
     * information
     *
     * @param properties Subscription information
     *
     * @return Subscription result (Successful:true, Failed:false)
     *
     * @endif
     */
    virtual bool subscribeInterface(const SDOPackage::NVList& properties);
    
    /*!
     * @if jp
     * @brief f[^Mʒm̓o^
     *
     * f[^Mʒm̎󂯎肩o^B
     *
     * @param properties o^
     *
     * @else
     * @brief Unsubscribe the data receive notification
     *
     * Unsubscribe the data receive notification.
     *
     * @param properties Unsubscription information
     *
     * @endif
     */
    virtual void unsubscribeInterface(const SDOPackage::NVList& properties);
    
  private:
    //    RTC::OutPortCdr_var m_outport;
    CdrBufferBase* m_buffer;
  };
};     // namespace RTC

extern "C"
{
  void OutPortCorbaCdrConsumerInit(void);
};

#endif // RTC_OUTPORTCORBACDRCONSUMER_H
