// -*- C++ -*-
/*!
 * @file SdoConfiguration.h
 * @brief RT component base class
 * @date $Date: 2008-01-14 07:49:34 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: SdoConfiguration.h 1243 2009-03-12 09:41:36Z openrtm $
 *
 */

#ifndef SdoConfiguration_h
#define SdoConfiguration_h

// CORBA header include
#include <rtm/RTC.h>
#include <coil/Mutex.h>
#include <coil/Guard.h>

// local includes
#include <rtm/idl/SDOPackageSkel.h>
#include <rtm/ConfigAdmin.h>
#include <rtm/SystemLogger.h>
#include <string>

// SdoConfiguration with SeqEx 159120
// SdoConfiguration with SeqUtil 114504 114224

/*!
 * @if jp
 * @namespace SDOPackage
 *
 * @brief SDO pbP[W
 *
 * @else
 *
 * @namespace SDOPackage
 *
 * @brief SDO Package
 *
 * @endif
 */

#ifdef WIN32
#pragma warning( disable : 4290 )
#endif

namespace SDOPackage
{
  /*!
   * @if jp
   *
   * @class Configuration_impl
   * @brief SDO Configuration NX
   *
   * Configuration interface  Resource Data Model Œ`ꂽf[^
   * ǉA폜̑s߂̃C^[tF[XłB
   * DeviceProfile, ServiceProfile, ConfigurationProfile  Organization
   * ̕ύXs߂̃Iy[VĂBSDO ̎dlł̓ANZX
   * уZLeBɊւڍׂɂĂ͋K肵ĂȂB
   * 
   * ̐ݒ (Configuration) ێ邱ƂɂAeՂfݒ
   * 𔽉f邱ƂłBOɒ`ꂽ̐ݒ ConfigurationSets
   *  configuration profile Ƃĕێ邱ƂłBЂƂ
   * ConfigurationSet ͓̐ݒɊ֘AtꂽSvpeBl̃XgA
   * j[NIDAڍׂƂƂɎĂBɂAeݒ荀ڂ̏ڍׂLq
   * ʂ邱ƂłBConfiguration interface ̃Iy[V͂
   * ConfiguratioinSets ̊ǗxB
   *
   *
   * - ConfigurationSet: id, description, NVList \1Zbg̐ݒ
   * - ConfigurationSetList: ConfigurationSet ̃Xg
   * - Parameter: name, type, allowed_values \p[^`B
   * - ActiveConfigurationSet: ݗLȃRtBM[V1ZbgB
   *
   * ȉASDOdlɖLĂȂ͉߂킩ȂߓƎ
   *
   * ȉ̊֐ ParameterList ɑ΂ďsB
   * - get_configuration_parameters()
   *
   * ȉ̊֐̓ANeBuConfigurationSetɑ΂鏈s
   * - get_configuration_parameter_values()
   * - get_configuration_parameter_value()
   * - set_configuration_parameter()
   *
   * ȉ̊֐ConfigurationSetListɑ΂ďs
   * - get_configuration_sets()
   * - get_configuration_set()
   * - set_configuration_set_values()
   * - get_active_configuration_set()
   * - add_configuration_set()
   * - remove_configuration_set()
   * - activate_configuration_set()
   *
   * @since 0.4.0
   *
   * @else
   *
   * @class Configuration_impl
   * @brief Configuration implementation class
   *
   * Configuration interface provides operations to add or remove data
   * specified in resource data model. These operations provide functions to
   * change DeviceProfile, ServiceProfile, ConfigurationProfile, and
   * Organization. This SDO specification does not address access control or
   * security aspects.
   *
   * Different configurations can be stored for simple and quick activation.
   * Different predefined configurations are stored as different
   * ConfigurationSets or configuration profile. A ConfigurationSet stores the
   * value of all properties assigned for the particular configuration along
   * with its unique id and description to identify and describe the
   * configuration respectively. Operations in the configuration interface
   * help manage these ConfigurationSets.
   *
   *
   * - ConfigurationSet: id, description, one configuration set to consist 
   *                     of NVList 
   * - ConfigurationSetList: List of ConfigurationSet
   * - Parameter: Parameter definition consist of name, type and allowed_values
   * - ActiveConfigurationSet: One set of configuration set that is valid.
   *
   * The following functions do processing to ParameterList. 
   * - get_configuration_parameters()
   *
   * The following functions do processing to active ConfigurationSet
   * - get_configuration_parameter_values()
   * - get_configuration_parameter_value()
   * - set_configuration_parameter()
   *
   * The following functions do processing to ConfigurationSetList
   * - get_configuration_sets()
   * - get_configuration_set()
   * - set_configuration_set_values()
   * - get_active_configuration_set()
   * - add_configuration_set()
   * - remove_configuration_set()
   * - activate_configuration_set()
   *
   * @since 0.4.0
   *
   * @endif
   */
  class Configuration_impl
    : public virtual POA_SDOPackage::Configuration,
      public virtual PortableServer::RefCountServantBase
  {
    typedef coil::Mutex Mutex;
    typedef coil::Guard<Mutex> Guard;
  public:
    /*!
     * @if jp
     *
     * @brief RXgN^
     * 
     * RXgN^
     *
     * @param configAdmin ConfigurationSetList
     * 
     * @else
     *
     * @brief Constructor
     * 
     * Constructor
     *
     * @param configAdmin ConfigurationSetList
     *
     * @endif
     */
    Configuration_impl(RTC::ConfigAdmin& configAdmin);
    
    /*!
     * @if jp
     *
     * @brief zfXgN^
     * 
     * zfXgN^B
     * 
     * @else
     *
     * @brief Virtual destractor
     *
     * Virtual destractor.
     
     * @endif
     */
    virtual ~Configuration_impl(void);
    
    //============================================================
    //
    // <<< CORBA interfaces >>>
    //
    //============================================================
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] SDO  DeviceProfile ̃Zbg
     *
     * ̃Iy[V SDO  DeviceProfile ZbgBSDO 
     * DeviceProfile ێĂȂꍇ͐V DeviceProfile 𐶐A
     * DeviceProfile łɕێĂꍇ͊̂̂ƒuB
     *
     * @param dProfile SDO Ɋ֘At DeviceProfileB
     *
     * @return Iy[VǂԂB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InvalidParameter  "dProfile"  null łB
     * @exception InternalError IG[B
     * 
     * @else
     *
     * @brief [CORBA interface] Set DeviceProfile of SDO
     *
     * This operation sets the DeviceProfile of an SDO. If the SDO does not
     * have DeviceProfile, the operation will create a new DeviceProfile,
     * otherwise it will replace the existing DeviceProfile.
     *
     * @param dProfile The device profile that is to be assigned to this SDO.
     *
     * @return If the operation was successfully completed.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InvalidParameter The argument "dProfile" is null.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean set_device_profile(const DeviceProfile& dProfile)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] SDO  ServiceProfile ̃Zbg
     *
     * ̃Iy[V͂ Configuration interface LΏ SDO 
     * ServiceProfile ǉB ServiceProfile  id ł
     * V ID ꂻ ServiceProfile i[B id 
     * Ȃ΁ASDO ͓ id  ServiceProfile B
     *  id ݂Ȃ΂ ServiceProfile ǉAid ݂
     * ㏑B<br>
     * (ӁFŐVo[Wł̓Iy[Vadd_service_profileύX)
     *
     * @param sProfile ǉ ServiceProfile
     *
     * @return Iy[VǂԂB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter  "sProfile"  nullłB
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Set SDO's ServiceProfile
     *
     * This operation adds ServiceProfile to the target SDO that navigates this
     * Configuration interface. If the id in argument ServiceProfile is null,
     * new id is created and the ServiceProfile is stored. If the id is not
     * null, the target SDO searches for ServiceProfile in it with the same id.
     * It adds the ServiceProfile if not exist, or overwrites if exist.
     *
     * @param sProfile ServiceProfile to be added.
     *
     * @return If the operation was successfully completed.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InvalidParameter The argument "sProfile" is null.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean add_service_profile(const ServiceProfile& sProfile)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] Organization ̒ǉ
     *
     * ̃Iy[V Organization object ̃t@XǉB
     *
     * @param org ǉ Organization
     *
     * @return Iy[VǂԂB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InvalidParameter  "organization"  null łB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Add Organization
     *
     * This operation adds reference of an Organization object.
     *
     * @param org Organization to be added.
     *
     * @return If the operation was successfully completed.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InvalidParameter The argument gorganizationh is null.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean add_organization(Organization_ptr org)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ServiceProfile ̍폜
     *
     * ̃Iy[V͂ Configuration interface  SDO 
     * Service  ServiceProfile 폜B폜 ServiceProfile
     * ͈ɂw肳B
     *
     * @param id 폜 ServiceProfile  serviceIDB
     *
     * @return Iy[VǂԂB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter  "id"  null łB "id" 
     *            ֘Atꂽ ServiceProfile ݂ȂB
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Remove ServiceProfile
     *
     * This operation removes ServiceProfile object to the SDO that has this
     * Configuration interface. The ServiceProfile object to be removed is
     * specified by argument.
     *
     * @param id serviceID of a ServiceProfile to be removed.
     *
     * @return If the operation was successfully completed.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception InvalidParameter The argument "sProfile" is null, or if the
     *                             object that is specified by argument
     *                             "sProfile" does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean remove_service_profile(const char* id)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] Organization ̎QƂ̍폜
     *
     * ̃Iy[V Organization ̎QƂ폜B
     *
     * @param organization_id 폜 Organization ̈ӂ idB
     *
     * @return Iy[VǂԂB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter  "organization_id"  null łB
     *                              "organization_id" Ɋ֘Atꂽ 
     *                             OrganizationProfile ݂ȂB
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Remove the reference of Organization 
     *
     * This operation removes the reference of an Organization object.
     *
     * @param organization_id Unique id of the organization to be removed.
     *
     * @return If the operation was successfully completed.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception InvalidParameter The argument "organizationID" is null,
     *                             or the object which is specified by argument
     *                             "organizationID" does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean remove_organization(const char* organization_id)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ݒp[^̃Xg̎擾
     *
     * ̃Iy[V configuration parameter ̃XgԂB
     * SDO ݒ\ȃp[^Ȃ΋̃XgԂB
     *
     * @return ݒtp[^`̃XgB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Get a list of configuration parameters
     *
     * This operation returns a list of Parameters. An empty list is returned
     * if the SDO does not have any configurable parameter.
     *
     * @return The list with definitions of parameters characterizing the
     *          configuration.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual ParameterList* get_configuration_parameters()
      throw (CORBA::SystemException,
	     NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] Configuration parameter ̒l̃Xg̎擾
     *
     * ̃Iy[V͑SĂ configuration p[^ђlԂB
     *
     * @return SĂ configuration p[^ƒl̃XgB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Get a list of the value of configuration 
     *                          parameters
     *
     * This operation returns all configuration parameters and their values.
     *
     * @return List of all configuration parameters and their values.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual NVList* get_configuration_parameter_values()
      throw (CORBA::SystemException,
	     NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] Configuration parameter ̒l̎擾
     *
     * ̃Iy[V͈ "name" Ŏw肳ꂽp[^lԂB
     *
     * @param name lvp[^̖OB
     *
     * @return w肳ꂽp[^̒lB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter  "name"  null łB
     *             "name" Ɋ֘Atꂽp[^݂ȂB
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Get the value of configuration parameter
     *
     * This operation returns a value of parameter that is specified by
     * argument "name."
     *
     * @param name Name of the parameter whose value is requested.
     *
     * @return The value of the specified parameter.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception InvalidParameter The value of the argument "name" is
     *                             empty String, or null, or the parameter
     *                             that is specified by argument "name"
     *                             does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Any* get_configuration_parameter_value(const char* name)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] Configuration p[^̕ύX
     *
     * ̃Iy[V "name" Ŏw肵p[^̒l "value" 
     * ύXB
     *
     * @param name ύXΏۃp[^̖OB
     * @param value ύXΏۃp[^̐VlB
     *
     * @return Iy[VǂԂB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter ( "name""value")  null łB
     *             "name" Ɋ֘Atꂽp[^݂ȂB
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Modify the configuration parameter value
     *
     * This operation sets a parameter to a value that is specified by argument
     * "value." The parameter to be modified is specified by argument " name."
     *
     * @param name The name of parameter to be modified.
     * @param value New value of the specified parameter.
     *
     * @return If the operation was successfully completed.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception InvalidParameter The arguments ("name" and/or "value") is
     *                             null, or the parameter that is specified by
     *                             the argument "name" does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean set_configuration_parameter(const char* name,
						       const CORBA::Any& value)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
        
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ConfigurationSet Xg̎擾 
     *
     * ̃Iy[V ConfigurationProfile  ConfigurationSet 
     * XgԂB SDO  ConfigurationSet Ȃ΋̃XgԂB
     *
     * @return ێĂ ConfigurationSet ̃XǧݒlB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Get a list of ConfigurationSet
     *
     * This operation returns a list of ConfigurationSets that the
     * ConfigurationProfile has. An empty list is returned if the SDO does not
     * have any ConfigurationSets.
     * This operation returns a list of all ConfigurationSets of the SDO.
     * If no predefined ConfigurationSets exist, then empty list is returned.
     *
     * @return The list of stored configuration with their current values.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual ConfigurationSetList* get_configuration_sets()
      throw (CORBA::SystemException,
	     NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ConfigurationSet ̎擾
     *
     * ̃Iy[V͈Ŏw肳ꂽ ConfigurationSet  ID Ɋ֘A
     * tꂽ ConfigurationSet ԂB
     *
     * @param config_id ConfigurationSet ̎ʎqB
     *
     * @return ɂw肳ꂽ ConfigurationSetB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter "config_id"  null Aw肳ꂽ
     *            ConfigurationSet ݂ȂB
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Get a ConfigurationSet
     *
     * This operation returns the ConfigurationSet specified by the parameter
     * configurationSetID.
     *
     * @param config_id Identifier of ConfigurationSet requested.
     *
     * @return The configuration set specified by the parameter config_id.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception InvalidParameter The parameter 'config_id' is null or 
     *                             there are no ConfigurationSets stored with
     *                             such id.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual ConfigurationSet* get_configuration_set(const char* config_id)
      throw (CORBA::SystemException,
	     NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ANeBu ConfigurationSet 擾
     *
     * ̃Iy[V͓YSDǑ݃ANeBu ConfigurationSet ԂB
     * (SDǑ݂̐ݒ肪\ߒ`ꂽ ConfigurationSet ɂݒ肳
     * Ȃ΁B)
     * ConfigurationSet ͈ȉ̏ꍇɂ̓ANeBuł͂Ȃ̂Ƃ݂ȂB
     *
     * - ݂̐ݒ肪\ߒ`ꂽ ConfigurationSet ɂZbgĂȂA
     * - SDO ̐ݒ肪ANeBuɂȂɕύXꂽA
     * - SDO ݒ肷 ConfigurationSet ύXꂽA
     * 
     * ̏ꍇɂ́A ConfigurationSet ԂB
     *
     * @return ݃ANeBu ConfigurationSetB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Get active ConfigurationSet
     *
     * This operation returns the current active ConfigurationSet of an
     * SDO (i.e., if the current configuration of the SDO was set using
     * predefined configuration set).
     * ConfigurationSet cannot be considered active if the:
     *
     * - Current configuration of the SDO was not set using any predefined
     *   ConfigurationSet, or
     * - Configuration of the SDO was changed after it has been active, or
     * - ConfigurationSet that was used to configure the SDO was modified.
     *
     * Empty ConfigurationSet is returned in these cases.
     *
     * @return The active ConfigurationSet.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual ConfigurationSet* get_active_configuration_set()
      throw (CORBA::SystemException,
	     NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ConfigurationSet ǉ
     *
     * ConfigurationProfile  ConfigurationSet ǉIy[VB
     *
     * @param configuration_set ǉ ConfigurationSetB
     *
     * @return Iy[VǂB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter "configurationSet"  null A
     *            "configurationSet"Œ`ꂽ̂PsA
     *            w肳ꂽ configurationSet IDɑ݂B
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Add ConfigurationSet
     *
     * This operation adds a ConfigurationSet to the ConfigurationProfile.
     *
     * @param configuration_set The ConfigurationSet that is added.
     *
     * @return If the operation was successfully completed.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception InvalidParameter The argument "configurationSet" is null,
     *                             or one of the attributes defining
     *                             "configurationSet" is invalid, or the 
     *                             specified identifier of the configuration
     *                             set already exists.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean
    add_configuration_set(const ConfigurationSet& configuration_set)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ConfigurationSet Zbg
     *
     * ̃Iy[V͎w肳ꂽ id  ConfigurationSet XVB
     *
     * @param config_id ύX ConfigurationSet  IDB
     * @param configuration_set ύX ConfigurationSet ̂́B
     *
     * @return ConfigurationSet ɍXVłꍇ trueB
     *         łȂ false ԂB
     *
     * @exception InvalidParameter config_id  null A
     *                             w肳ꂽ id Ŋi[ꂽ ConfigurationSet
     *                             ݂ȂAw肳ꂽ configuration_set
     *                             ̑̂PsB
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Set ConfigurationSet
     *
     * This operation modifies the specified ConfigurationSet of an SDO.
     *
     * Note: The number of parameters differs between spec and IDL!!
     *
     * @param config_id The ID of ConfigurationSet to be modified.
     * @param configuration_set ConfigurationSet to be replaced.
     *
     * @return A flag indicating if the ConfigurationSet was modified 
     *         successfully. "true" - The ConfigurationSet was modified
     *         successfully. "false" - The ConfigurationSet could not be
     *         modified successfully.
     *
     * @exception InvalidParameter The parameter 'configurationSetID' is null
     *                             or there is no ConfigurationSet stored with 
     *                             such id.
     *                             This exception is also raised if one of the 
     *                             attributes defining ConfigurationSet is not 
     *                             valid.
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean
    set_configuration_set_values(const ConfigurationSet& configuration_set)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ConfigurationSet 폜
     *
     * ConfigurationProfile  ConfigurationSet 폜B
     *
     * @param config_id 폜 ConfigurationSet  idB
     *
     * @return Iy[VǂB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter  "configurationSetID"  null łA
     *            ́AŎw肳ꂽ ConfigurationSet ݂ȂB
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Remove ConfigurationSet
     *
     * This operation removes a ConfigurationSet from the ConfigurationProfile.
     *
     * @param config_id The id of ConfigurationSet which is removed.
     *
     * @return If the operation was successfully completed.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception InvalidParameter The arguments "configurationSetID" is null,
     *                             or the object specified by the argument
     *                             "configurationSetID" does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean remove_configuration_set(const char* config_id)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ConfigurationSet ̃ANeBu
     *
     * ConfigurationProfile Ɋi[ꂽ ConfigurationSet ̂
     * ANeBuɂB
     * ̃Iy[V͓ ConfigurationSet ANeBuɂB
     * Ȃ킿ASDO ̃RtBM[VEvpeB̊i[Ă
     * ConfigurationSet ɂݒ肳vpeB̒lɕύXB
     * w肳ꂽ ConfigurationSet ̒lANeBuERtBM[V
     * ɃRs[ƂƂӖB
     *
     * @param config_id ANeBu ConfigurationSet  idB
     *
     * @return Iy[VǂB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter  "config_id"  null łA
     *            Ŏw肳ꂽ ConfigurationSet ݂ȂB
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Activate ConfigurationSet
     *
     * This operation activates one of the stored ConfigurationSets in the
     * ConfigurationProfile.
     * This operation activates the specified stored ConfigurationSets.
     * This means that the configuration properties of the SDO are changed as
     * the values of these properties specified in the stored ConfigurationSet.
     * In other words, values of the specified ConfigurationSet are now copied
     * to the active configuration.
     *
     * @param config_id Identifier of ConfigurationSet to be activated.
     *
     * @return If the operation was successfully completed.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception InvalidParameter The argument ("configID") is null or
     *                             there is no configuration set with identifier
     *                             specified by the argument.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean activate_configuration_set(const char* config_id)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    // end of CORBA interface definition
    //============================================================
    
    /*!
     * @if jp
     *
     * @brief IuWFNg@t@X擾
     * 
     * Ώۂ̃IuWFNgt@X擾
     * 
     * @return IuWFNgt@X
     * 
     * @else
     *
     * @brief Get object reference
     * 
     * Get the target object reference.
     * 
     * @return Object reference
     * 
     * @endif
     */
    Configuration_ptr getObjRef();
    
    /*!
     * @if jp
     *
     * @brief SDO  DeviceProfile 擾
     * 
     * SDO  DeviceProfile 擾
     * 
     * @return SDO  DeviceProfile
     * 
     * @else
     *
     * @brief Get the DeviceProfile of SDO
     * 
     * Get the DeviceProfile of SDO.
     * 
     * @return DeviceProfile of SDO
     * 
     * @endif
     */
    const DeviceProfile getDeviceProfile();
    
    /*!
     * @if jp
     *
     * @brief SDO  ServiceProfile ̃Xg擾
     * 
     * SDO  ServiceProfile ̃Xg擾
     * 
     * @return SDO ServiceProfileXg
     * 
     * @else
     *
     * @brief Get a list of ServiceProfile of SDO
     * 
     * Get a list of ServiceProfile of SDO.
     * 
     * @return List of ServiceProfile of SDO
     * 
     * @endif
     */
    const ServiceProfileList getServiceProfiles();
    
    /*!
     * @if jp
     *
     * @brief SDO  ServiceProfile 擾
     * 
     * ̃Iy[V͈ "id" Ŏw肳ꂽSDO  ServiceProfileԂB
     * "id" Ŏw肳ꂽ ServiceProfile݂ȂꍇA
     * ServiceProfilẽCX^X𐶐ԂB
     * 
     * @param id ServiceProfile ̎ʎqB
     * 
     * @return w肳ꂽ SDO ServiceProfile
     * 
     * @else
     *
     * @brief Get Service Profile of SDO
     * 
     * This operation returns ServiceProfile of SDO specified by argument "id".
     * If ServiceProfile specified by "id" does not exist,
     * the instance of ServiceProfile will be generated and returned.
     * 
     * @param id Identifier of ServiceProfile
     * 
     * @return The specified SDO ServiceProfile
     * 
     * @endif
     */
    const ServiceProfile getServiceProfile(const char* id);
    
    /*!
     * @if jp
     *
     * @brief SDO  Organization Xg擾
     * 
     * SDO  Organization Xg擾
     * 
     * @return SDO  Organization Xg
     * 
     * @else
     *
     * @brief Get a list of Organization of SDO
     * 
     * Get a list of Organization of SDO.
     * 
     * @return List of Organization of SDO
     * 
     * @endif
     */
    const OrganizationList getOrganizations();
    
  protected:
    ::RTC::Logger rtclog;
    /*!
     * @if jp
     *
     * @brief UUID𐶐
     * 
     * UUID𐶐
     * 
     * @return UUID
     * 
     * @else
     *
     * @brief Generate UUID
     * 
     * Generate UUID.
     * 
     * @return UUID that has been generated
     * 
     * @endif
     */
    const std::string getUUID() const;
    
    /*!
     * @if jp
     * @brief CORBA IuWFNgւ̎Q
     * @else
     * @brief The reference to CORBA object
     * @endif
     */
    Configuration_var m_objref;
    
    /*!
     * @if jp
     * @brief Lock t SDO DeviceProfile
     * @else
     * @brief SDO DeviceProfile with mutex lock
     * @endif
     */
    DeviceProfile m_deviceProfile;
    Mutex m_dprofile_mutex;
    
    /*!
     * @if jp
     * @brief Lock t SDO ServiceProfileList
     * @else
     * @brief SDO ServiceProfileList with mutex lock
     * @endif
     */
    ServiceProfileList m_serviceProfiles;
    Mutex m_sprofile_mutex;
    
    /*!
     * @if jp
     * @brief SDO Parameter
     * 
     * Zpɔˑȕϐ(p[^)`f[^\B
     * p[^\́Aϐ̖Oƌ^`B
     * `Ă鑮͈ȉ̂ƂB
     *  - name : p[^̖OB
     *  - type : p[^̌^Bp[^Ef[^^̃IWi̒l͈͂
     *            allowedValues ̒`Ō肷邱ƂłB
     * - allowedValues : p[^邱Ƃ̂łlB
     *                   p[^^ɌŗL̒`肷Kvꍇ̂
     *                   ͎̑gpBႦ΁Ap[^ɋ
     *                   l񋓂ɂČ肵Al^p[^ɋ
     *                   l͈͂ɂČ肵肷Bp[^ɋ
     *                   ĺA񋓁A͈͂܂̓C^[o\Œ`
     *                   邱ƂłB
     *                   p[^ɑ΂鐧񂪂Ȃꍇ́AallowedValues
     *                   nullƂȂBȂ킿Ap[^^ɌŗL͈̔
     *                   ł΂ǂ̂悤Ȓl邱ƂłB
     * 
     * @else
     * @brief SDO Parameter
     * 
     * Data structure to define a variable (parameter) independently 
     * of implementation technologies. The Parameter structure defines 
     * the name and type of a variable.
     * Attributes defined in Parameter.
     *  - name : Parameterfs name.
     *  - type : Name of parameter's type. The original value scope of 
     *           parameter data type can be constrained by definitions 
     *           allocated in the attribute allowedValues.
     * - allowedValues : Values that the parameter can accept.
     *           This attribute is used only when the value scope
     *           inherent to the parameter type must be constrained. For
     *           example, the values allowed for a string parameter may
     *           be constrained by an enumeration, or the values
     *           allowed for a numeric parameter may be constrained by
     *           a range. The values allowed for a parameter can be
     *           defined in enumeration, range, or interval structures.
     *           The value of attribute allowedValues is null if there is
     *           no constraint on a parameter value, that is, any value
     *           can be assigned to the parameter as far as it follows the
     *           value scope inherent to the parameterfs type.
     * @endif
     *
     *    struct Parameter
     *    {
     *      string         name;
     *      TypeCode  type;
     *      AllowedValues allowed_values;
     *    };
     */

    /*!
     * @if jp
     * @brief Lock t SDO ParameterList
     * @else
     * @brief SDO ParameterList with mutex lock
     * @endif
     */
    ParameterList m_parameters;
    Mutex m_params_mutex;
    
    /*!
     * @if jp
     * @brief Lock t SDO ConfigurationSetList
     * @else
     * @brief SDO ConfigurationSetList with mutex lock
     * @endif
     */
    /*
      struct ConfigurationSet
      {
      string id;
      string description;
      NVList configuration_data;
      };
    */
    RTC::ConfigAdmin& m_configsets;
    Mutex m_config_mutex;
    
    /*!
     * @if jp
     * @brief  Lock t SDO OrganizationList
     * @else
     * @brief SDO OrganizationList with mutex lock
     * @endif
     */
    OrganizationList m_organizations;
    Mutex m_org_mutex;
    
    /*!
     * @if jp
     * @brief  NVListpfunctor
     * @else
     * @brief  Functor for NVList
     * @endif
     */
    struct nv_name
    {
      nv_name(const char* name) : m_name(name) {};
      bool operator()(const NameValue& nv)
      {
	return m_name == std::string(nv.name);
      }
      std::string m_name;
    };
    
    /*!
     * @if jp
     * @brief  ServiceProfilepfunctor
     * @else
     * @brief  Functor for ServiceProfile
     * @endif
     */
    struct service_id
    {
      service_id(const char* id) : m_id(id) {};
      bool operator()(const ServiceProfile& s)
      {
	std::string id(s.id);
	return m_id == id;
      }
      const std::string m_id;
    };
    
    /*!
     * @if jp
     * @brief  Organizationpfunctor
     * @else
     * @brief  Functor for Organization
     * @endif
     */
    struct org_id
    {
      org_id(const char* id) : m_id(id) {};
      bool operator()(const Organization_ptr& o)
      {
	std::string id(o->get_organization_id());
	return m_id == id;
      }
      const std::string m_id;
    };
    
    /*!
     * @if jp
     * @brief  ConfigurationSetpfunctor
     * @else
     * @brief  Functor for ConfigurationSet
     * @endif
     */
    struct config_id
    {
      config_id(const char* id) :  m_id(id) {};
      bool operator()(const ConfigurationSet& c)
      {
	std::string id(c.id);
	return m_id == id;
      }
      const std::string m_id;
    };
  };  // class Configuration_impl
}; // namespace SDOPackage

#ifdef WIN32
#pragma warning( default : 4290 )
#endif

#endif // SdoConfiguration_h
