// -*- C++ -*-
/*!
 * @file SdoOrganization.h
 * @brief SDO Organization implementation class
 * @date $Date: 2008-01-14 07:49:28 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: SdoOrganization.h 1243 2009-03-12 09:41:36Z openrtm $
 *
 */

#ifndef SdoOrganization_h
#define SdoOrganization_h

#include <rtm/RTC.h>
#include <rtm/idl/SDOPackageSkel.h>
#include <rtm/SystemLogger.h>
#include <string>
#include <coil/Mutex.h>
#include <coil/Guard.h>

/*!
 * @if jp
 * @namespace SDOPackage
 *
 * @brief SDO pbP[W
 *
 * @else
 *
 * @namespace SDOPackage
 *
 * @brief SDO Package
 *
 * @endif
 */

#ifdef WIN32
#pragma warning( disable : 4290 )
#endif

namespace SDOPackage
{
  /*!
   * @if jp
   *
   * @class Organization_impl
   * @brief SDO Organization NX
   *
   * Organization interface  Resource Data Model Œ`ꂽf[^
   * ǉA폜̑s߂̃C^[tF[XłB
   *
   * @since 0.4.0
   *
   * @else
   *
   * @class Organization_impl
   * @brief Organization implementation class
   *
   * Organization interface is an interface to add and delete etc data
   * defined by Resource Data Model.
   *
   * @since 0.4.0
   *
   * @endif
   */
  class Organization_impl
    : public virtual POA_SDOPackage::Organization,
      public virtual PortableServer::RefCountServantBase
  {
    typedef coil::Mutex Mutex;
    typedef coil::Guard<Mutex> Guard;
  public:
    /*!
     * @if jp
     *
     * @brief RXgN^
     * 
     * RXgN^
     *
     * @else
     *
     * @brief Constructor
     * 
     * Constructor
     *
     * @endif
     */
    Organization_impl(SDOSystemElement_ptr sdo);
    
    /*!
     * @if jp
     *
     * @brief zfXgN^
     * 
     * zfXgN^B
     * 
     * @else
     *
     * @brief Virtual destructor
     * 
     * Virtual Virtual destructor
     *
     * @endif
     */
    virtual ~Organization_impl(void);
    
    //============================================================
    //
    // <<< CORBA interfaces >>>
    //
    //============================================================
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] Organization ID 擾
     *
     * Organization  ID ԂIy[VB
     *
     * @return Resource Data Model Œ`ꂽ Organization IDB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Get Organization ID
     *
     * This operation returns the 'ID' of the Organization.
     *
     * @return The id of the Organization defined in the resource data model.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual char* get_organization_id()
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] OrganizationProperty ̃Zbg
     *
     *  SDO Specification  PIM LqƃIy[VقȂB
     *  addOrganizationProperty ɑΉH<BR>
     * OrganizationProperty  Organization ɒǉIy[VB
     * OrganizationProperty  Organization ̃vpeBLqłB
     *
     * @param org_property Zbg OrganizationProperty
     *
     * @return Iy[VǂԂB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter "org_property"  nullB
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Set OrganizationProperty
     *
     * Note: The PIM description of SDO Specification differs from the operation
     *       name.
     * Note: Does this operation correspond to addOrganizationProperty?
     * This operation adds the OrganizationProperty to an Organization. The
     * OrganizationProperty is the property description of an Organization.
     *
     * @param org_property The type of organization to be added.
     *
     * @return If the operation was successfully completed.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception InvalidParameter The argument "organizationProperty" is null.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean
    add_organization_property(const OrganizationProperty& org_property)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] OrganizationProperty ̎擾
     *
     * Organization L OrganizationProperty ԂIy[VB
     * Organization vpeBȂ΋̃XgԂB
     *
     * @return Organization ̃vpeB̃XgB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Get OrganizationProperty
     *
     * This operation returns the OrganizationProperty that an Organization
     * has. An empty OrganizationProperty is returned if the Organization does
     * not have any properties.
     *
     * @return The list with properties of the organization.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual OrganizationProperty* get_organization_property()
      throw (CORBA::SystemException,
	     NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] OrganizationProperty ̓̒l̎擾
     *
     * OrganizationProperty ̎w肳ꂽlԂIy[VB
     *  "name" Ŏw肳ꂽvpeB̒lԂB
     *
     * @param name lԂvpeB̖OB
     *
     * @return  "name" Ŏw肳ꂽvpeB̒lB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter  "name" Ŏw肳ꂽvpeB
     *            ݂ȂB
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Get specified value of OrganizationProperty
     *
     * This operation returns a value in the OrganizationProperty.
     * The value to be returned is specified by argument "name."
     *
     * @param name The name of the value to be returned.
     *
     * @return The value of property which is specified by argument "name".
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception InvalidParameter There are no Property stored with argument
     *                             "name".
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Any* get_organization_property_value(const char* name)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] OrganizationProperty ̒l̃Zbg
     *
     * OrganizationProperty  NVList  name  value ̃Zbgǉ
     * XVIy[VBname  value ͈ "name"  "value" ɂ
     * w肷B
     *
     * @param name ǉEXVvpeB̖OB
     * @param value ǉEXVvpeB̒lB
     *
     * @return Iy[VǂԂB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter  "name" Ŏw肳ꂽvpeB
     *            ݂ȂB
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Set specified value of OrganizationProperty
     *
     * This operation adds or updates a pair of name and value as a property
     * of Organization to/in NVList of the OrganizationProperty. The name and
     * the value to be added/updated are specified by argument "name" and
     * "value."
     *
     * @param name The name of the property to be added/updated.
     * @param value The value of the property to be added/updated.
     *
     * @return If the operation was successfully completed.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InvalidParameter The property that is specified by argument
     *                             "name" does not exist.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean
    set_organization_property_value(const char* name, const CORBA::Any& value)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] OrganizationProperty ̍폜
     *
     * OrganizationProperty  NVList ̃vpeB폜B
     * 폜vpeB̖O͈ "name" ɂw肳B
     *
     * @param name 폜vpeB̖OB
     *
     * @return Iy[VǂԂB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter  "name" Ŏw肳ꂽvpeB
     *            ݂ȂB
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Remove specified OrganizationProperty
     *
     * This operation removes a property of Organization from NVList of the
     * OrganizationProperty. The property to be removed is specified by
     * argument "name."
     *
     * @param name The name of the property to be removed.
     *
     * @return If the operation was successfully completed.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InvalidParameter The property that is specified by argument
     *                             "name" does not exist.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean remove_organization_property(const char* name)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] SDO o[̒ǉ
     *
     * Organization Ƀo[Ƃ SDO ǉB
     *  "sdo" ɒǉ郁o[ SDO w肷B
     *
     * @param sdo_list Organization ɒǉ SDO ̃XgB
     *
     * @return Iy[VǂԂB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter  "sdo"  nullłB
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Add the member SDOs
     *
     * This operation adds a member that is an SDO to the organization.
     * The member to be added is specified by argument "sdo."
     *
     * @param sdo_list The member to be added to the organization.
     *
     * @return If the operation was successfully completed.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InvalidParameter The argument "sdo" is null.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean add_members(const SDOList& sdo_list)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] Organization ̃o[擾
     *
     * Organization ̃o[ SDO ̃XgԂB
     * o[݂Ȃ΋̃XgԂB
     *
     * @return Organization Ɋ܂܂郁o[ SDO ̃XgB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Get the member list of the Organization
     *
     * This operation returns a list of SDOs that are members of an
     * Organization. An empty list is returned if the Organization does not
     * have any members.
     *
     * @return Member SDOs that are contained in the Organization object.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual SDOList* get_members()
      throw (CORBA::SystemException,
	     NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] SDO  Zbg
     *
     * SDO ̃Xg Organization ̃o[ƂăZbgB
     * Organization łɃo[ SDO ǗĂꍇ́A
     * ^ꂽ SDO ̃XgɒuB
     *
     * @param sdos o[ SDOB
     *
     * @return Iy[VǂԂB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter  "SDOList"  nullłA
     *            Ɏw肳ꂽ "SDOList" ݂ȂB
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Set SDO
     *
     * This operation assigns a list of SDOs to an Organization as its members.
     * If the Organization has already maintained a member SDO(s) when it is
     * called, the operation replaces the member(s) with specified list of
     * SDOs.
     *
     * @param sdos Member SDOs to be assigned.
     *
     * @return If the operation was successfully completed.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InvalidParameter The argument "SDOList" is null, or the object
     *                             that is specified by the argument "sdos"
     *                             does not exist.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean set_members(const SDOList& sdos)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] SDO o[̍폜
     *
     * Organization Ŏw肳ꂽ "id"  SDO 폜B
     *
     * @param id 폜 SDO  idB
     *
     * @return Iy[VǂԂB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter  "id"  null ݂͑ȂB
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Remove member SDO from Organization
     *
     * This operation removes a member from the organization. The member to be
     * removed is specified by argument "id."
     *
     * @param id Id of the SDO to be removed from the organization.
     *
     * @return If the operation was successfully completed.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InvalidParameter The argument "id" is null or does not exist.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean remove_member(const char* id)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] Organization ̃I[i[擾
     *
     *  Organization ̃I[i[ւ̎QƂԂB
     *
     * @return I[i[IuWFNgւ̎QƁB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Get the owner of Organization
     *
     * This operation returns the SDOSystemElement that is owner of
     * the Organization.
     *
     * @return Reference of owner object.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual SDOSystemElement_ptr get_owner()
      throw (CORBA::SystemException,
	     NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] Organization ɃI[i[Zbg
     *
     * Organization ɑ΂ SDOSystemElement I[i[ƂăZbgB
     *  "sdo" ɃZbg SDOSystemElement w肷B
     *
     * @param sdo I[i[IuWFNg̎QƁB
     *
     * @return Iy[VǂԂB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter  "sdo"  nullłÁA
     *                             "sdo" ݂ȂB
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Set the owner to the Organization
     *
     * This operation sets an SDOSystemElement to the owner of the
     * Organization. The SDOSystemElement to be set is specified by argument
     * "sdo."
     *
     * @param sdo Reference of owner object.
     *
     * @return If the operation was successfully completed.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InvalidParameter The argument "sdo" is null, or the object
     *                             that is specified by "sdo" in argument "sdo"
     *                             does not exist.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean set_owner(SDOSystemElement_ptr sdo)
      throw (CORBA::SystemException,
	     InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] Organization  DependencyType 擾
     *
     * Organization ̊֌W\ "DependencyType" ԂB
     *
     * @return Organization ̈ˑ֌W DependencyType ԂB
     *         DependencyType  OMG SDO dl Section 2.2.2 2-3 y[W
     *         "Data Structures Used by Resource Data Model" QƁB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Get the DependencyType of the Organization
     *
     * This operation gets the relationship "DependencyType" of the
     * Organization.
     *
     * @return The relationship of the Organization as DependencyType.
     *         DependencyType is defined in Section 2.2.2, "Data Structures
     *         Used by Resource Data Model," on page 2-3
     *         of OMG SDO Specification.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual DependencyType get_dependency()
      throw (CORBA::SystemException,
	     NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] Organization  DependencyType Zbg
     *
     * Organization ̈ˑ֌W "DependencyType" ZbgB
     *  "dependency" ɂˑ֌W^B
     *
     * @param dependency Organization ̈ˑ֌W\ DependencyTypeB
     *        DependencyType  OMG SDO dl Section 2.2.2A2-3 y[W
     *        "Data Structures Used by Resource Data Model" QƁB
     *
     * @return Iy[VǂԂB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception InvalidParameter  "sProfile"  nullłB
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [CORBA interface] Set the DependencyType of the Organization
     *
     * This operation sets the relationship "DependencyType" of the
     * Organization. The value to be set is specified by argument "dependency."
     *
     * @param dependency The relationship of the Organization as
     *                   DependencyType. DependencyType is defined in Section
     *                   2.2.2, "Data Structures Used by Resource Data Model,"
     *                   on page 2-3.
     *
     * @return If the operation was successfully completed.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InvalidParameter The argument "dependency" is null.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean set_dependency(DependencyType dependency)
      throw (CORBA::SystemException,
	     NotAvailable, InternalError);
    
    // end of CORBA interface definition
    //============================================================
    Organization_ptr getObjRef() {return m_objref;};

  protected:
    ::RTC::Logger rtclog;
    Organization_var m_objref;
    /*!
     * @if jp
     * @brief Organization ̎ʎq
     * @else
     * @brief The identifier of the Organization.
     * @endif
     */
    char* m_pId;
    
    /*!
     * @if jp
     * @brief Organization Ɋ֘Atꂽ SDO õXg
     * @else
     * @brief A list of SDO members associated with the Organization
     * @endif
     */
    SDOPackage::SDOList m_memberList;
    
    /*!
     * @if jp
     * @brief Organization  owner
     * @else
     * @brief The owner of the Organization
     * @endif
     */   
    SDOPackage::SDOSystemElement_var m_varOwner;
    
    /*!
     * @if jp
     * 
     * @brief ˑ֌W̃^Cv
     *
     * Owner  member ̈ˑ֌Ww肷鑮B
     * Organization ͈ȉ̃g|Wp^[\邱ƂłB
     *
     * -# owner  member ǗKwI\B̏ꍇ DependencyType  OWN
     *    ƂlB
     * -# members  owner Ǘt̊KwI\B̏ꍇ
     *    DependencyType  OWNER ƂlB
     * -# owner  member Ɉˑ֌WȂtbgȍ\B̏ꍇ
     *    DependencyType  NO_DEPENDENCY ƂlB
     * 
     * SDO  SDOSystemElement ̃TuNX Organization  owner Ƃ
     * UƂoBSDO  owner ̏ꍇɂOrganization ͏L
     * ꂩ̃g|W[p^[ƂB
     *
     * - Organization  1. ̃g|W[p^[ꍇAB owner SDO
     *    member SDO 𐧌䂷BƂ΁AGAR(owner)́A
     *   xZT(member)AxZT(member)Aʐ(member)𐧌䂷B
     * - Organization  2. ̃g|Wꍇ́A SDO member B
     *   SDO owner LBƂ΁AAv(owner)͂AV
     *   R|[lg(member)狤LB
     * - SDO ł͂Ȃ SDOSystemElement ̃TuNX owner ̏ꍇA
     *   ȉ̂悤ȃg|W[ႪlB
     * -- User(owner)-SDO(member): [U(owner) ͈ȏ SDO(member)
     *   ǗB͏Lg|Wp^1.ɂB
     * -- Location(owner)-SDO(members): ȏ SDO(member) ̏ꏊ
     *   = location(owner) œ삵ĂꍇAOrganization ̃g|Wp^[
     *    3. ̏ꍇɂȂBƂ΁A PDA 镔ɂA݂ɓ
     *   Ȋ֌Wł葊݂ɒʐM\ȏꍇ͂ɂB
     *
     * @else
     *
     * @brief Dependency type
     *
     * This attribute specifies the dependency relation between the owner and
     * members of the organization.
     * Organization is used to form the following three patterns of topology.
     *
     * -# Hierarchical organization, which indicates owner supervises members.
     *    In this case, DependencyType should hold OWN value (see description
     *    of DependencyType on previous pages).
     * -# Reversely hierarchical organization, which indicates members
     *    supervise owner. In this case, DependencyType should hold OWNED value
     *    (see description of DependencyType on previous pages).
     * -# Flat organization, which indicates no dependency exists. In this
     *    case, DependencyType should hold NO_DEPENDENCY value (see description
     *    of DependencyType on previous pages).
     *
     * Both an SDO and another subclass of SDOSystemElement can act as owner
     * of an Organization. When an SDO is an owner, Organization can represent
     * any of the above three topology patterns.
     *
     * - When an Organization represents topology pattern (1), an SDO (owner)
     *   controls one or more SDOs (members). For example, air conditioner
     *   (owner) controls a temperature sensor (member), humidity sensor
     *   (member), and wind flow controller (member).
     * - When an Organization represents topology pattern (2), multiple
     *   SDOs(members) share an SDO (owner). For example, an amplifier (owner)
     *   is shared by several AV components (members) in an AV stereo.
     * - When a subclass of SDOSystemElement, which is not an SDO is an owner
     *   examples of the topology are as follows.
     * -- User (owner)-SDO (members): When a user (owner) supervises one or
     *    more SDOs (members), the organization represents topology pattern 1.
     * -- Location (owner)-SDO (members): When one or more SDOs (members) are
     *    operating in a specific location (owner), the organization represents
     *    topology pattern 3. For example, multiple PDAs in the same place
     *    (e.g., a room) have equal relationships among them to communicate
     *    with each other.
     * @endif
     */
    SDOPackage::DependencyType m_dependency;
    
    /*!
     * @if jp
     *
     * @brief Organization vpeB
     *
     * OrganizationProperty  Organization ̃vpeBێB
     *  Organization 01 OrganizationProperty B
     *
     * @else
     *
     * @brief Organization property
     *
     * OrganizationProperty contains the properties of an Organization.
     * An Organization has zero or one (at most one) instance
     * of OrganizationProperty.
     *
     * @endif
     */
    SDOPackage::OrganizationProperty m_orgProperty;
    Mutex m_org_mutex;
    
    /*!
     * @if jp
     * @brief  NameValuepfunctor
     * @else
     * @brief  Functor for NameValue
     * @endif
     */
    struct nv_name
    {
      nv_name(const char* name) : m_name(name) {};
      bool operator()(const NameValue& nv)
      {
	return m_name == std::string(nv.name);
      }
      std::string m_name;
    };  // struct nv_name
    
    /*!
     * @if jp
     * @brief  SDOpfunctor
     * @else
     * @brief  Functor for SDO
     * @endif
     */
    struct sdo_id
    {
      sdo_id(const char* id) : m_id(id) {};
      bool operator()(const SDO_ptr sdo)
      {
        CORBA::String_var id(sdo->get_sdo_id());
	return m_id == (const char*)id;
      }
      std::string m_id;
    };  // struct sdo_id
  };  // class Organization_impl
};  // namespace SDOPackage

#ifdef WIN32
#pragma warning( default : 4290 )
#endif

#endif // namespace SDOPackage
